#version 460
 #extension GL_EXT_multiview : enable
#define VIEW gl_ViewIndex
#extension GL_EXT_scalar_block_layout : require

struct CascadeLayer
{
    vec2 m_xWorldPos;
    vec2 m_xWorldPosShadow;
    float m_fWorldSize;
    float m_fMinWavelength;
    vec2 _padding;
};

struct WaterMaterialParams
{
    vec3 m_xSubsurfaceScatterColour;
    float m_fRoughnessFactor;
    vec3 m_xSubsurfaceScatterColourFastFlow;
    float m_fCausticsStrength_Unused;
    vec4 m_xReflectionColour;
    vec3 m_xFogTransmissionAlpha;
    float m_fNormalMapStrength;
    float m_fEdgeBlendRadius;
    uint m_bShowOnMap;
    float m_fFoamDissipationRate;
    float m_fFoamWaveCoverage;
    vec4 m_xFoamColour;
    float m_fFoamWaveGenerationRate;
    float m_fFoamFlowAccelerationThreshold;
    float m_fFoamFlowDecelerationThreshold;
    float m_fFoamFlowGenerationRate;
    float m_fFoamCoastlineGenerationRate;
    float m_fFoamParticleCount;
    float m_fFoamParticleMinCoverage;
    float m_fFoamParticleMaxCoverage;
    float m_fFoamParticleSizeScale;
    float m_fFoamParticleMovementSpeed;
    float m_fParticleShadowStrength;
    float m_fFoamParticleSpecularStrength;
    float m_fSplashVelocityScale;
    float m_fSplashMinSimAcceleration;
    float m_fSplashMaxSimAcceleration;
    float m_fSplashSizeScale;
    float m_fSplashParticleCount;
    float m_fSplashSpecular;
    float m_fCollisionParticleGenerationRate;
    float m_fCollisionParticleStaticJumpSpeed;
    float m_fCollisionParticleWaveJumpSpeed;
    float m_fCollisionParticleSize;
    float m_fCollisionParticleGroundSpeedScale;
    float m_fCollisionParticleBorderThickness;
    float m_fCollisionParticleSpecular;
    vec3 m_xMaterialParamsPadding0;
    vec4 m_xFoamParticleSizeOverLife;
    vec4 m_xSplashParticleSizeOverLife;
    vec4 m_xCollisionParticleSizeOverLife;
};

struct WaterMeshInstanceData
{
    vec3 m_xWorldMeshPosition;
    float m_fWorldMeshScale;
    uint m_uCascadeIndex;
    float m_fWorldVertexSize;
    float m_fWorldTexelSize;
    float m_fPadding;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 3, binding = 3, std140) uniform type_CascadeDefinition
{
    CascadeLayer m_xLayer[10];
    int m_iLayerCount;
    float m_fResolution;
    float g_fAltitudeAlpha;
    int g_iCascadeShiftSinceLastUpdate;
} CascadeDefinition;

layout(set = 3, binding = 5, std140) uniform type_WaterDataTextureParams
{
    vec4 m_xWorldToUVScaleAndOffset;
    vec2 m_xTileLookupUVScale;
    vec2 m_xInputUVScale;
    vec2 m_xTiledUVScale;
    vec2 m_xWetnessMaskTileLookupUVScale;
    vec2 m_xWetnessMaskInputUVScale;
    vec2 m_xWetnessMaskTiledUVScale;
    vec2 m_xTileClassificationBoundsMin;
    vec2 m_xOneOverTileClassificationBoundsRange;
    int m_iTileClassificationMipOffset;
    float m_fWaterLevelMin;
    float m_fWaterLevelMax;
    float m_fOpenWaterLevel;
    uint m_uOpenWaterMaterialID;
    uint m_uOpenWaterSpectrumID;
    uint m_uLevelHasOpenWater;
    uint m_uPadding;
} WaterDataTextureParams;

layout(set = 3, binding = 62, scalar) readonly buffer type_StructuredBuffer_WaterMaterialParams
{
    WaterMaterialParams _m0[];
} g_xWaterMaterialData;

layout(set = 4, binding = 72, std430) readonly buffer type_StructuredBuffer_WaterMeshInstanceData
{
    WaterMeshInstanceData _m0[];
} g_axWaterMeshInstanceData;

layout(set = 3, binding = 12, std140) uniform type_WaterMiscParams
{
    vec3 g_xDetailCenter;
    uint g_uMinQuadMeshCascade;
    vec2 g_xNormalMapScrollSpeeds1;
    vec2 g_xNormalMapScrollSpeeds2;
    vec2 g_xNormalMapScales;
} WaterMiscParams;

layout(set = 0, binding = 1) uniform sampler g_xPointClamp;
layout(set = 0, binding = 3) uniform sampler g_xBilinearClamp;
layout(set = 0, binding = 7) uniform sampler g_xFilteredClamp;
layout(set = 3, binding = 60) uniform texture2D g_xWaterTileLookupTexture;
layout(set = 3, binding = 61) uniform texture2D g_xMaterialMapTexture;
layout(set = 3, binding = 64) uniform texture2D g_xWaterLevelTexture;
layout(set = 3, binding = 67) uniform texture2D g_xWaterFlowVelocityTexture;
layout(set = 3, binding = 63) uniform texture2DArray g_xGerstnerTexture;
layout(set = 3, binding = 66) uniform texture2DArray g_xSimulationFoamTexture;
layout(set = 3, binding = 74) uniform texture2DArray g_xShadowTexture;

layout(location = 0) in vec4 in_var_POSITION0;
layout(location = 0) out vec4 out_var_WorldSpacePos_Distance;
layout(location = 1) out float out_var_RoughnessFactor;
layout(location = 2) out vec4 out_var_ReflectionColour_WaterLevelDx;
layout(location = 3) out vec4 out_var_FogTransmissionAlpha_WaterLevelDz;
layout(location = 4) out vec4 out_var_OriginalWorldPos_LodAlpha;
layout(location = 5) out vec4 out_var_FlowVelocity_FoamSim_Shadow;
layout(location = 6) out vec4 out_var_SubsurfaceScatterColour_NormalMapStrength;
layout(location = 7) out vec4 out_var_SubsurfaceScatterColourFastFlow_EdgeBlendRadius;
layout(location = 8) flat out uint out_var_InstanceId;
layout(location = 9) out vec4 out_var_FoamColour;

vec4 _129;
vec4 _130;

void main()
{
    vec2 _143 = in_var_POSITION0.xz * g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_fWorldMeshScale;
    vec3 _146 = vec4(_143.x, in_var_POSITION0.y, _143.y, in_var_POSITION0.w).xyz + g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_xWorldMeshPosition;
    vec3 _147 = _146.xyz;
    float _148 = _146.x;
    float _167 = min(clamp((max(abs(_148 - WaterMiscParams.g_xDetailCenter.x), abs(_146.z - WaterMiscParams.g_xDetailCenter.z)) / g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_fWorldMeshScale) - 3.0, 0.0, 1.0) + ((g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_uCascadeIndex == 0u) ? CascadeDefinition.g_fAltitudeAlpha : 0.0), 1.0);
    float _168 = 2.0 * g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_fWorldVertexSize;
    vec3 _177;
    if (fract(_148 / _168) > 0.25)
    {
        vec3 _176 = _147;
        _176.x = _148 - (g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_fWorldVertexSize * _167);
        _177 = _176;
    }
    else
    {
        _177 = _147;
    }
    vec3 _187;
    if (fract(_177.z / _168) > 0.25)
    {
        vec3 _186 = _177;
        _186.z = _177.z - (g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_fWorldVertexSize * _167);
        _187 = _186;
    }
    else
    {
        _187 = _177;
    }
    uint _190 = min((g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_uCascadeIndex + 1u), 7u);
    int _192 = int(g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_uCascadeIndex);
    vec2 _198 = vec2(CascadeDefinition.m_xLayer[_192].m_fWorldSize);
    float _200 = float(g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_uCascadeIndex);
    vec3 _203 = vec3((_187.xz - CascadeDefinition.m_xLayer[_192].m_xWorldPos) / _198, _200);
    int _204 = int(_190);
    vec2 _210 = vec2(CascadeDefinition.m_xLayer[_204].m_fWorldSize);
    float _212 = float(_190);
    vec3 _215 = vec3((_187.xz - CascadeDefinition.m_xLayer[_204].m_xWorldPos) / _210, _212);
    vec4 _235;
    if (g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_uCascadeIndex < WaterMiscParams.g_uMinQuadMeshCascade)
    {
        vec3 _233 = _187.xyz + (textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), _203 + vec3(0.0, 0.0, 1.0), 0.0).xyz * (((g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_uCascadeIndex >= 6u) ? CascadeDefinition.g_fAltitudeAlpha : 1.0) * (1.0 - _167)));
        _235 = vec4(_233.x, _233.y, _233.z, in_var_POSITION0.w);
    }
    else
    {
        _235 = vec4(_187.x, _187.y, _187.z, in_var_POSITION0.w);
    }
    vec4 _252;
    if (_190 < WaterMiscParams.g_uMinQuadMeshCascade)
    {
        vec3 _250 = _235.xyz + (textureLod(sampler2DArray(g_xGerstnerTexture, g_xFilteredClamp), _215 + vec3(0.0, 0.0, 1.0), 0.0).xyz * (((_190 >= 6u) ? CascadeDefinition.g_fAltitudeAlpha : 1.0) * _167));
        _252 = vec4(_250.x, _250.y, _250.z, _235.w);
    }
    else
    {
        _252 = _235;
    }
    vec2 _265;
    bool _285;
    vec2 _286;
    do
    {
        _265 = WaterDataTextureParams.m_xWorldToUVScaleAndOffset.xy;
        vec2 _268 = (_252.xz * _265) + WaterDataTextureParams.m_xWorldToUVScaleAndOffset.zw;
        if (all(lessThan(abs(_268 - vec2(0.5)), vec2(0.5))))
        {
            _285 = true;
            _286 = ((_268 * WaterDataTextureParams.m_xInputUVScale) + (textureLod(sampler2D(g_xWaterTileLookupTexture, g_xPointClamp), clamp(_268, vec2(0.0), vec2(1.0)) * WaterDataTextureParams.m_xTileLookupUVScale, 0.0).xy * 32767.0)) * WaterDataTextureParams.m_xTiledUVScale;
            break;
        }
        _285 = false;
        _286 = vec2(0.0);
        break;
    } while(false);
    bool _289;
    float _304;
    do
    {
        _289 = _285 == false;
        if (_289)
        {
            _304 = WaterDataTextureParams.m_fOpenWaterLevel;
            break;
        }
        _304 = mix(WaterDataTextureParams.m_fWaterLevelMin, WaterDataTextureParams.m_fWaterLevelMax, textureLod(sampler2D(g_xWaterLevelTexture, g_xBilinearClamp), _286, 0.0).x);
        break;
    } while(false);
    vec4 _307 = _252;
    _307.y = _252.y + _304;
    vec2 _309 = vec2(0.5) / (_265 * WaterDataTextureParams.m_xInputUVScale);
    vec2 _310 = WaterDataTextureParams.m_xTiledUVScale * 0.5;
    float _330;
    do
    {
        if (_289)
        {
            _330 = WaterDataTextureParams.m_fOpenWaterLevel;
            break;
        }
        _330 = mix(WaterDataTextureParams.m_fWaterLevelMin, WaterDataTextureParams.m_fWaterLevelMax, textureLod(sampler2D(g_xWaterLevelTexture, g_xBilinearClamp), _286 + vec2(_310.x, 0.0), 0.0).x);
        break;
    } while(false);
    float _350;
    do
    {
        if (_289)
        {
            _350 = WaterDataTextureParams.m_fOpenWaterLevel;
            break;
        }
        _350 = mix(WaterDataTextureParams.m_fWaterLevelMin, WaterDataTextureParams.m_fWaterLevelMax, textureLod(sampler2D(g_xWaterLevelTexture, g_xBilinearClamp), _286 + vec2(0.0, _310.y), 0.0).x);
        break;
    } while(false);
    vec4 _357 = _130;
    _357.w = (_330 - _304) / _309.x;
    vec4 _358 = _130;
    _358.w = (_350 - _304) / _309.y;
    vec4 _360 = vec4(_187.x, _187.y, _187.z, _129.w);
    _360.w = _167;
    vec4 _370 = _130;
    _370.z = mix(textureLod(sampler2DArray(g_xSimulationFoamTexture, g_xFilteredClamp), _203, 0.0).x, textureLod(sampler2DArray(g_xSimulationFoamTexture, g_xFilteredClamp), _215, 0.0).x, _167);
    vec3 _533;
    float _534;
    vec3 _535;
    vec4 _536;
    vec3 _537;
    float _538;
    float _539;
    vec4 _540;
    do
    {
        if (_289)
        {
            _533 = g_xWaterMaterialData._m0[WaterDataTextureParams.m_uOpenWaterMaterialID].m_xSubsurfaceScatterColour;
            _534 = g_xWaterMaterialData._m0[WaterDataTextureParams.m_uOpenWaterMaterialID].m_fRoughnessFactor;
            _535 = g_xWaterMaterialData._m0[WaterDataTextureParams.m_uOpenWaterMaterialID].m_xSubsurfaceScatterColourFastFlow;
            _536 = g_xWaterMaterialData._m0[WaterDataTextureParams.m_uOpenWaterMaterialID].m_xReflectionColour;
            _537 = g_xWaterMaterialData._m0[WaterDataTextureParams.m_uOpenWaterMaterialID].m_xFogTransmissionAlpha;
            _538 = g_xWaterMaterialData._m0[WaterDataTextureParams.m_uOpenWaterMaterialID].m_fNormalMapStrength;
            _539 = g_xWaterMaterialData._m0[WaterDataTextureParams.m_uOpenWaterMaterialID].m_fEdgeBlendRadius;
            _540 = g_xWaterMaterialData._m0[WaterDataTextureParams.m_uOpenWaterMaterialID].m_xFoamColour;
            break;
        }
        else
        {
            uvec4 _383 = uvec4(textureLod(sampler2D(g_xMaterialMapTexture, g_xPointClamp), _286, 0.0) * 255.0);
            uvec4 _385 = _383 >> (uvec4(4u) & uvec4(31u));
            vec4 _388 = vec4(_383 & uvec4(15u)) * vec4(0.066666670143604278564453125);
            uint _389 = _385.x;
            float _390 = _388.x;
            uint _415 = _385.y;
            float _416 = _388.y;
            uint _449 = _385.z;
            float _450 = _388.z;
            uint _483 = _385.w;
            float _484 = _388.w;
            _533 = (((g_xWaterMaterialData._m0[_389].m_xSubsurfaceScatterColour * _390) + (g_xWaterMaterialData._m0[_415].m_xSubsurfaceScatterColour * _416)) + (g_xWaterMaterialData._m0[_449].m_xSubsurfaceScatterColour * _450)) + (g_xWaterMaterialData._m0[_483].m_xSubsurfaceScatterColour * _484);
            _534 = (((g_xWaterMaterialData._m0[_389].m_fRoughnessFactor * _390) + (g_xWaterMaterialData._m0[_415].m_fRoughnessFactor * _416)) + (g_xWaterMaterialData._m0[_449].m_fRoughnessFactor * _450)) + (g_xWaterMaterialData._m0[_483].m_fRoughnessFactor * _484);
            _535 = (((g_xWaterMaterialData._m0[_389].m_xSubsurfaceScatterColourFastFlow * _390) + (g_xWaterMaterialData._m0[_415].m_xSubsurfaceScatterColourFastFlow * _416)) + (g_xWaterMaterialData._m0[_449].m_xSubsurfaceScatterColourFastFlow * _450)) + (g_xWaterMaterialData._m0[_483].m_xSubsurfaceScatterColourFastFlow * _484);
            _536 = (((g_xWaterMaterialData._m0[_389].m_xReflectionColour * _390) + (g_xWaterMaterialData._m0[_415].m_xReflectionColour * _416)) + (g_xWaterMaterialData._m0[_449].m_xReflectionColour * _450)) + (g_xWaterMaterialData._m0[_483].m_xReflectionColour * _484);
            _537 = (((g_xWaterMaterialData._m0[_389].m_xFogTransmissionAlpha * _390) + (g_xWaterMaterialData._m0[_415].m_xFogTransmissionAlpha * _416)) + (g_xWaterMaterialData._m0[_449].m_xFogTransmissionAlpha * _450)) + (g_xWaterMaterialData._m0[_483].m_xFogTransmissionAlpha * _484);
            _538 = (((g_xWaterMaterialData._m0[_389].m_fNormalMapStrength * _390) + (g_xWaterMaterialData._m0[_415].m_fNormalMapStrength * _416)) + (g_xWaterMaterialData._m0[_449].m_fNormalMapStrength * _450)) + (g_xWaterMaterialData._m0[_483].m_fNormalMapStrength * _484);
            _539 = (((g_xWaterMaterialData._m0[_389].m_fEdgeBlendRadius * _390) + (g_xWaterMaterialData._m0[_415].m_fEdgeBlendRadius * _416)) + (g_xWaterMaterialData._m0[_449].m_fEdgeBlendRadius * _450)) + (g_xWaterMaterialData._m0[_483].m_fEdgeBlendRadius * _484);
            _540 = (((g_xWaterMaterialData._m0[_389].m_xFoamColour * _390) + (g_xWaterMaterialData._m0[_415].m_xFoamColour * _416)) + (g_xWaterMaterialData._m0[_449].m_xFoamColour * _450)) + (g_xWaterMaterialData._m0[_483].m_xFoamColour * _484);
            break;
        }
    } while(false);
    vec4 _542 = vec4(_533.x, _533.y, _533.z, _129.w);
    _542.w = _538;
    vec4 _544 = vec4(_535.x, _535.y, _535.z, _129.w);
    _544.w = _539;
    vec4 _557;
    if (_285)
    {
        vec2 _555 = textureLod(sampler2D(g_xWaterFlowVelocityTexture, g_xBilinearClamp), _286, 0.0).xy * 7.0;
        _557 = vec4(_555.x, _555.y, _370.z, _370.w);
    }
    else
    {
        _557 = vec4(vec2(0.0).x, vec2(0.0).y, _370.z, _370.w);
    }
    vec4 _582 = _557;
    _582.w = 1.0 - mix(textureLod(sampler2DArray(g_xShadowTexture, g_xFilteredClamp), vec3((_307.xz - CascadeDefinition.m_xLayer[g_axWaterMeshInstanceData._m0[uint(gl_InstanceIndex)].m_uCascadeIndex].m_xWorldPosShadow) / _198, _200), 0.0).x, textureLod(sampler2DArray(g_xShadowTexture, g_xFilteredClamp), vec3((_307.xz - CascadeDefinition.m_xLayer[_190].m_xWorldPosShadow) / _210, _212), 0.0).x, _167);
    vec4 _585 = _307 * FrameConstants.g_xViewProj;
    vec4 _587 = _130;
    _587.w = _585.w;
    _585.x += vk3d_params[VIEW].stereo.x * (_585.w - vk3d_params[VIEW].stereo.y);
    gl_Position = _585;

// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
//gl_Position.x += vk3d_params[VIEW].stereo.x * (gl_Position.w - vk3d_params[VIEW].stereo.y);
    out_var_WorldSpacePos_Distance = vec4(_307.x, _307.y, _307.z, _587.w);
    out_var_RoughnessFactor = _534;
    out_var_ReflectionColour_WaterLevelDx = vec4(_536.x, _536.y, _536.z, _357.w);
    out_var_FogTransmissionAlpha_WaterLevelDz = vec4(_537.x, _537.y, _537.z, _358.w);
    out_var_OriginalWorldPos_LodAlpha = _360;
    out_var_FlowVelocity_FoamSim_Shadow = _582;
    out_var_SubsurfaceScatterColour_NormalMapStrength = _542;
    out_var_SubsurfaceScatterColourFastFlow_EdgeBlendRadius = _544;
    out_var_InstanceId = uint(gl_InstanceIndex);
    out_var_FoamColour = _540;
}


