// Lights
#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct PhotonFace
{
    uint uLevel;
    float fSize;
    float fInvSize;
    int iShadowIndex;
};

struct PhotonParams
{
    mat4 m_xLightOrientation;
    mat4 m_xCustomVolumeMatrix;
    vec3 m_xLightPos;
    float m_fAreaLightRadius;
    vec3 m_xLightDir;
    float m_fLightRange;
    vec3 m_xLightColour;
    uint m_bUseLimitAngles;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fLightAngleOuter;
    float m_fRcpLightAngleRange;
    float m_fArcCosLimitAngle;
    float m_fRcpProjCubeLerpDist;
    uint m_bUseProjCube;
    uint m_bUseDualProjCube;
    vec2 m_xDepthProjection;
    float m_fLightMeshRadius;
    float m_fShadowStrength;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fPadding;
    int m_iProjCubeTextureIndex;
    int m_iProjCubeBlurredTextureIndex;
    PhotonFace m_axFace[6];
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 4, std140) uniform type_PhotonIndex
{
    uint g_uPhotonIndex;
} PhotonIndex;

layout(set = 1, binding = 89, std430) readonly buffer type_StructuredBuffer_PhotonParams
{
    layout(row_major) PhotonParams _m0[];
} g_axPhotons;

layout(set = 0, binding = 5) uniform sampler g_xTrilinearClamp;
layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xLighting_AlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xLighting_PBRPropertiesBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;
layout(set = 1, binding = 54) uniform textureCube g_xProjCubeMap;
layout(set = 1, binding = 55) uniform textureCube g_xProjCubeMapBlurred;

layout(location = 0) out vec4 out_var_SV_Target;

vec3 _135;

void main()
{
    ivec2 _144 = ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy;
    float _152 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _144, 0).x);
    //vec3 _171 = (vec4((FrameConstants.g_xCameraTanHalfFOV * _152) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _152, 1.0) * FrameConstants.g_xInvView).xyz;
    
     vec4 temp = vec4((FrameConstants.g_xCameraTanHalfFOV * _152) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy)
     - vec2(1.0)), _152, 1.0);
 
     temp.x -= vk3d_params[VIEW].stereo.x * (_152- vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0];
     
    vec3 _171 = (temp * FrameConstants.g_xInvView).xyz;
    
    vec4 _173 = TEXEL_FETCH(g_xLighting_NormalBuffer, _144, 0);
    vec4 _175 = TEXEL_FETCH(g_xLighting_AlbedoBuffer, _144, 0);
    vec4 _177 = TEXEL_FETCH(g_xLighting_PBRPropertiesBuffer, _144, 0);
    vec3 _181 = normalize((_173.xyz * 2.0) - vec3(1.0));
    vec3 _182 = _175.xyz;
    float _183 = _177.y;
    float _184 = _175.w;
    float _185 = _177.x;
    vec2 _188 = (_177.zw * 2.0) - vec2(1.0);
    vec2 _190 = _188.xy;
    float _194 = sqrt(clamp(1.0 - dot(_190, _190), 0.0, 1.0));
    vec3 _199 = vec3(_188.x, _188.y, _135.z);
    _199.z = (_173.w != 0.0) ? _194 : (-_194);
    vec4 _548;
    bool _549;
    do
    {
        vec3 _241 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightPos - _171;
        float _242 = length(_241);
        vec3 _244 = _241 / vec3(_242);
        float _252 = pow(clamp((1.0 - clamp((_242 - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius) / (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRange - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius), 0.0, 1.0)) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightFalloffScale, 0.0, 1.0), g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightFalloffPower);
        float _260;
        if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_bUseLimitAngles != 0u)
        {
            _260 = _252 * clamp((dot(_244, g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightDir) + g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightAngleOuter) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fRcpLightAngleRange, 0.0, 1.0);
        }
        else
        {
            _260 = _252;
        }
        if (_260 < 9.9999997473787516355514526367188e-05)
        {
            _548 = vec4(0.0, 0.0, 0.0, 1.0);
            _549 = false;
            break;
        }
        vec3 _264 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightColour * _260;
        vec3 _291;
        if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_bUseProjCube != 0u)
        {
            vec3 _274 = mat3(g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightOrientation[0].xyz, g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightOrientation[1].xyz, g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightOrientation[2].xyz) * _244;
            vec4 _277 = texture(samplerCube(g_xProjCubeMap, g_xTrilinearClamp), _274);
            vec3 _278 = _277.xyz;
            vec3 _289;
            if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_bUseDualProjCube != 0u)
            {
                _289 = mix(_278, texture(samplerCube(g_xProjCubeMapBlurred, g_xTrilinearClamp), _274).xyz, vec3(min((_242 / g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRange) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fRcpProjCubeLerpDist, 1.0)));
            }
            else
            {
                _289 = _278;
            }
            _291 = _264 * _289;
        }
        else
        {
            _291 = _264;
        }
        vec3 _293 = normalize(FrameConstants.g_xCamPos - _171);
        float _311;
        vec3 _312;
        if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius > 0.0)
        {
            vec3 _298 = -reflect(_293, _181);
            vec3 _301 = (_298 * dot(_241, _298)) - _241;
            _311 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius * inversesqrt(dot(_241, _241));
            _312 = normalize(_241 + (_301 * clamp(g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius / length(_301), 0.0, 1.0)));
        }
        else
        {
            _311 = 0.0;
            _312 = _244;
        }
        float _314 = (_184 != 0.01960784383118152618408203125) ? _185 : 0.0;
        vec3 _316 = normalize(_293 + _312);
        float _318 = clamp(dot(_181, _244), 0.0, 1.0);
        float _320 = clamp(dot(_181, _293), 0.0, 1.0);
        float _322 = clamp(dot(_181, _316), 0.0, 1.0);
        float _324 = clamp(dot(_293, _316), 0.0, 1.0);
        bool _325 = _184 == 0.01568627543747425079345703125;
        vec3 _362;
        vec3 _363;
        vec3 _364;
        if (_325)
        {
            _362 = vec3(0.046999998390674591064453125);
            _363 = vec3(_318);
            _364 = _182;
        }
        else
        {
            vec3 _358;
            vec3 _359;
            vec3 _360;
            if (_184 == 0.007843137718737125396728515625)
            {
                _358 = vec3(0.0280000008642673492431640625);
                _359 = vec3(_318);
                _360 = _182;
            }
            else
            {
                vec3 _354;
                vec3 _355;
                vec3 _356;
                if (_184 == 0.0039215688593685626983642578125)
                {
                    _354 = vec3(0.0280000008642673492431640625);
                    _355 = vec3((clamp((clamp(dot(_244, -_181), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_244, -_293), 0.0, 1.0), 7.0), 0.0, 1.0) * _314) + _318);
                    _356 = _182;
                }
                else
                {
                    vec3 _337 = vec3(_314);
                    _354 = mix(vec3(0.039999999105930328369140625), _182, _337);
                    _355 = vec3(_318);
                    _356 = mix(_182, vec3(0.0), _337);
                }
                _358 = _354;
                _359 = _355;
                _360 = _356;
            }
            _362 = _358;
            _363 = _359;
            _364 = _360;
        }
        vec3 _365 = _363 * _364;
        float _366 = max(_183, 0.0500000007450580596923828125);
        float _367 = _366 * _366;
        vec3 _480;
        if (_325)
        {
            float _418 = clamp(1.0 - _183, 0.0, 1.0);
            float _420 = ((_185 * 2.0) - 1.0) * 0.300000011920928955078125;
            float _426 = pow(_418, 8.0) * 500.0;
            float _429 = pow(_418, 6.0) * 2000.0;
            float _443 = dot(normalize(_199 + (_181 * ((-0.0500000007450580596923828125) + _420))), _316);
            float _463 = dot(normalize(_199 + (_181 * ((0.0500000007450580596923828125 + _420) - (_418 * 0.0500000007450580596923828125)))), _316);
            _480 = _365 + ((((vec3(((smoothstep(-1.0, 0.0, _443) * pow(sqrt(clamp(1.0 - (_443 * _443), 0.0, 1.0)), 20.0 + _426)) * ((((_185 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _418))) * ((_426 + 28.0) * 0.03978873789310455322265625)) * (_182 / vec3(max(dot(_182, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _463) * pow(sqrt(clamp(1.0 - (_463 * _463), 0.0, 1.0)), 50.0 + _429)) * ((((_185 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _418)) * ((_429 + 58.0) * 0.03978873789310455322265625))) * _318) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fSpecularStrength);
        }
        else
        {
            float _373 = clamp(_311 * (1.0 - _367), 0.0, 1.0);
            float _393;
            if (_373 > 0.0)
            {
                float _384 = _367 * _367;
                _393 = _384 / (_384 + (((_373 * 0.25) * ((3.0 * _367) + _373)) / (_324 + 0.001000000047497451305389404296875)));
            }
            else
            {
                _393 = 1.0;
            }
            float _394 = 2.0 - _367;
            float _401 = _367 * _367;
            float _405 = (((_322 * _401) - _322) * _322) + 1.0;
            _480 = _365 + (((((vec3(1.0) - _362) * exp2((((-5.554729938507080078125) * _324) - 6.9831600189208984375) * _324)) + _362) * ((((1.0 / (((_320 * _394) + _367) * ((_318 * _394) + _367))) * (_401 * (1.0 / (_405 * _405)))) * _318) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fSpecularStrength)) * _393);
        }
        vec3 _530;
        if (_184 == 0.01960784383118152618408203125)
        {
            bool _484 = _185 > 0.4980392158031463623046875;
            vec3 _502 = sqrt(abs(_364));
            float _516 = 0.5 / _367;
            _530 = mix(_480, (_365 + (((((vec3(1.0) - _502) * exp2((((-5.554729938507080078125) * _324) - 6.9831600189208984375) * _324)) + _502) * ((1.0 / ((4.0 * ((_318 + _320) - (_318 * _320))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _516) * pow(1.0 - (_322 * _322), _516)))) * (_318 * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fSpecularStrength))) + (_364 * (clamp((clamp(dot(_244, -_181), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_244, -_293), 0.0, 1.0), 7.0), 0.0, 1.0) * (_484 ? 0.5 : 0.0))), vec3((_484 ? (_185 - 0.501960813999176025390625) : _185) * 2.007874011993408203125));
        }
        else
        {
            _530 = _480;
        }
        vec3 _533 = min(_530 * _291, vec3(1000.0)).xyz;
        vec3 _546;
        if ((_184 == 0.007843137718737125396728515625) && (_152 < 3.0))
        {
            _546 = _533 * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(_152 - 2.0, 0.0)));
        }
        else
        {
            _546 = _533;
        }
        _548 = vec4(_546.x, _546.y, _546.z, vec4(0.0, 0.0, 0.0, 1.0).w);
        _549 = true;
        break;
    } while(false);
    if (false == _549)
    {
        discard;
    }
    out_var_SV_Target = _548;
}


