// Lights
#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct PhotonFace
{
    uint uLevel;
    float fSize;
    float fInvSize;
    int iShadowIndex;
};

struct PhotonParams
{
    mat4 m_xLightOrientation;
    mat4 m_xCustomVolumeMatrix;
    vec3 m_xLightPos;
    float m_fAreaLightRadius;
    vec3 m_xLightDir;
    float m_fLightRange;
    vec3 m_xLightColour;
    uint m_bUseLimitAngles;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fLightAngleOuter;
    float m_fRcpLightAngleRange;
    float m_fArcCosLimitAngle;
    float m_fRcpProjCubeLerpDist;
    uint m_bUseProjCube;
    uint m_bUseDualProjCube;
    vec2 m_xDepthProjection;
    float m_fLightMeshRadius;
    float m_fShadowStrength;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fPadding;
    int m_iProjCubeTextureIndex;
    int m_iProjCubeBlurredTextureIndex;
    PhotonFace m_axFace[6];
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 4, std140) uniform type_PhotonIndex
{
    uint g_uPhotonIndex;
} PhotonIndex;

layout(set = 1, binding = 89, std430) readonly buffer type_StructuredBuffer_PhotonParams
{
    layout(row_major) PhotonParams _m0[];
} g_axPhotons;

layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xLighting_AlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xLighting_PBRPropertiesBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;

layout(location = 0) out vec4 out_var_SV_Target;

vec3 _123;

void main()
{
    ivec2 _132 = ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy;
    float _140 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _132, 0).x);
    //vec3 _159 = (vec4((FrameConstants.g_xCameraTanHalfFOV * _140) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _140, 1.0) * FrameConstants.g_xInvView).xyz;
    
    vec4 temp = vec4((FrameConstants.g_xCameraTanHalfFOV * _140) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy)
     - vec2(1.0)), _140, 1.0);
 
     temp.x -= vk3d_params[VIEW].stereo.x * (_140 - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0];
     
     vec3 _159 = (temp * FrameConstants.g_xInvView).xyz;
     
    vec3 _238 = (temp * FrameConstants.g_xInvView).xyz;
    vec4 _161 = TEXEL_FETCH(g_xLighting_NormalBuffer, _132, 0);
    vec4 _163 = TEXEL_FETCH(g_xLighting_AlbedoBuffer, _132, 0);
    vec4 _165 = TEXEL_FETCH(g_xLighting_PBRPropertiesBuffer, _132, 0);
    vec3 _169 = normalize((_161.xyz * 2.0) - vec3(1.0));
    vec3 _170 = _163.xyz;
    float _171 = _165.y;
    float _172 = _163.w;
    float _173 = _165.x;
    vec2 _176 = (_165.zw * 2.0) - vec2(1.0);
    vec2 _178 = _176.xy;
    float _182 = sqrt(clamp(1.0 - dot(_178, _178), 0.0, 1.0));
    vec3 _187 = vec3(_176.x, _176.y, _123.z);
    _187.z = (_161.w != 0.0) ? _182 : (-_182);
    vec4 _497;
    bool _498;
    do
    {
        vec3 _217 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightPos - _159;
        float _218 = length(_217);
        vec3 _220 = _217 / vec3(_218);
        float _228 = pow(clamp((1.0 - clamp((_218 - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius) / (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRange - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius), 0.0, 1.0)) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightFalloffScale, 0.0, 1.0), g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightFalloffPower);
        float _236;
        if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_bUseLimitAngles != 0u)
        {
            _236 = _228 * clamp((dot(_220, g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightDir) + g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightAngleOuter) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fRcpLightAngleRange, 0.0, 1.0);
        }
        else
        {
            _236 = _228;
        }
        if (_236 < 9.9999997473787516355514526367188e-05)
        {
            _497 = vec4(0.0, 0.0, 0.0, 1.0);
            _498 = false;
            break;
        }
        vec3 _242 = normalize(FrameConstants.g_xCamPos - _159);
        float _260;
        vec3 _261;
        if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius > 0.0)
        {
            vec3 _247 = -reflect(_242, _169);
            vec3 _250 = (_247 * dot(_217, _247)) - _217;
            _260 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius * inversesqrt(dot(_217, _217));
            _261 = normalize(_217 + (_250 * clamp(g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius / length(_250), 0.0, 1.0)));
        }
        else
        {
            _260 = 0.0;
            _261 = _220;
        }
        float _263 = (_172 != 0.01960784383118152618408203125) ? _173 : 0.0;
        vec3 _265 = normalize(_242 + _261);
        float _267 = clamp(dot(_169, _220), 0.0, 1.0);
        float _269 = clamp(dot(_169, _242), 0.0, 1.0);
        float _271 = clamp(dot(_169, _265), 0.0, 1.0);
        float _273 = clamp(dot(_242, _265), 0.0, 1.0);
        bool _274 = _172 == 0.01568627543747425079345703125;
        vec3 _311;
        vec3 _312;
        vec3 _313;
        if (_274)
        {
            _311 = vec3(0.046999998390674591064453125);
            _312 = vec3(_267);
            _313 = _170;
        }
        else
        {
            vec3 _307;
            vec3 _308;
            vec3 _309;
            if (_172 == 0.007843137718737125396728515625)
            {
                _307 = vec3(0.0280000008642673492431640625);
                _308 = vec3(_267);
                _309 = _170;
            }
            else
            {
                vec3 _303;
                vec3 _304;
                vec3 _305;
                if (_172 == 0.0039215688593685626983642578125)
                {
                    _303 = vec3(0.0280000008642673492431640625);
                    _304 = vec3((clamp((clamp(dot(_220, -_169), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_220, -_242), 0.0, 1.0), 7.0), 0.0, 1.0) * _263) + _267);
                    _305 = _170;
                }
                else
                {
                    vec3 _286 = vec3(_263);
                    _303 = mix(vec3(0.039999999105930328369140625), _170, _286);
                    _304 = vec3(_267);
                    _305 = mix(_170, vec3(0.0), _286);
                }
                _307 = _303;
                _308 = _304;
                _309 = _305;
            }
            _311 = _307;
            _312 = _308;
            _313 = _309;
        }
        vec3 _314 = _312 * _313;
        float _315 = max(_171, 0.0500000007450580596923828125);
        float _316 = _315 * _315;
        vec3 _429;
        if (_274)
        {
            float _367 = clamp(1.0 - _171, 0.0, 1.0);
            float _369 = ((_173 * 2.0) - 1.0) * 0.300000011920928955078125;
            float _375 = pow(_367, 8.0) * 500.0;
            float _378 = pow(_367, 6.0) * 2000.0;
            float _392 = dot(normalize(_187 + (_169 * ((-0.0500000007450580596923828125) + _369))), _265);
            float _412 = dot(normalize(_187 + (_169 * ((0.0500000007450580596923828125 + _369) - (_367 * 0.0500000007450580596923828125)))), _265);
            _429 = _314 + ((((vec3(((smoothstep(-1.0, 0.0, _392) * pow(sqrt(clamp(1.0 - (_392 * _392), 0.0, 1.0)), 20.0 + _375)) * ((((_173 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _367))) * ((_375 + 28.0) * 0.03978873789310455322265625)) * (_170 / vec3(max(dot(_170, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _412) * pow(sqrt(clamp(1.0 - (_412 * _412), 0.0, 1.0)), 50.0 + _378)) * ((((_173 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _367)) * ((_378 + 58.0) * 0.03978873789310455322265625))) * _267) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fSpecularStrength);
        }
        else
        {
            float _322 = clamp(_260 * (1.0 - _316), 0.0, 1.0);
            float _342;
            if (_322 > 0.0)
            {
                float _333 = _316 * _316;
                _342 = _333 / (_333 + (((_322 * 0.25) * ((3.0 * _316) + _322)) / (_273 + 0.001000000047497451305389404296875)));
            }
            else
            {
                _342 = 1.0;
            }
            float _343 = 2.0 - _316;
            float _350 = _316 * _316;
            float _354 = (((_271 * _350) - _271) * _271) + 1.0;
            _429 = _314 + (((((vec3(1.0) - _311) * exp2((((-5.554729938507080078125) * _273) - 6.9831600189208984375) * _273)) + _311) * ((((1.0 / (((_269 * _343) + _316) * ((_267 * _343) + _316))) * (_350 * (1.0 / (_354 * _354)))) * _267) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fSpecularStrength)) * _342);
        }
        vec3 _479;
        if (_172 == 0.01960784383118152618408203125)
        {
            bool _433 = _173 > 0.4980392158031463623046875;
            vec3 _451 = sqrt(abs(_313));
            float _465 = 0.5 / _316;
            _479 = mix(_429, (_314 + (((((vec3(1.0) - _451) * exp2((((-5.554729938507080078125) * _273) - 6.9831600189208984375) * _273)) + _451) * ((1.0 / ((4.0 * ((_267 + _269) - (_267 * _269))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _465) * pow(1.0 - (_271 * _271), _465)))) * (_267 * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fSpecularStrength))) + (_313 * (clamp((clamp(dot(_220, -_169), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_220, -_242), 0.0, 1.0), 7.0), 0.0, 1.0) * (_433 ? 0.5 : 0.0))), vec3((_433 ? (_173 - 0.501960813999176025390625) : _173) * 2.007874011993408203125));
        }
        else
        {
            _479 = _429;
        }
        vec3 _482 = min(_479 * (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightColour * _236), vec3(1000.0)).xyz;
        vec3 _495;
        if ((_172 == 0.007843137718737125396728515625) && (_140 < 3.0))
        {
            _495 = _482 * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(_140 - 2.0, 0.0)));
        }
        else
        {
            _495 = _482;
        }
        _497 = vec4(_495.x, _495.y, _495.z, vec4(0.0, 0.0, 0.0, 1.0).w);
        _498 = true;
        break;
    } while(false);
    if (false == _498)
    {
        discard;
    }
    out_var_SV_Target = _497;
}


