// Ligths
#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct PhotonFace
{
    uint uLevel;
    float fSize;
    float fInvSize;
    int iShadowIndex;
};

struct PhotonParams
{
    mat4 m_xLightOrientation;
    mat4 m_xCustomVolumeMatrix;
    vec3 m_xLightPos;
    float m_fAreaLightRadius;
    vec3 m_xLightDir;
    float m_fLightRange;
    vec3 m_xLightColour;
    uint m_bUseLimitAngles;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fLightAngleOuter;
    float m_fRcpLightAngleRange;
    float m_fArcCosLimitAngle;
    float m_fRcpProjCubeLerpDist;
    uint m_bUseProjCube;
    uint m_bUseDualProjCube;
    vec2 m_xDepthProjection;
    float m_fLightMeshRadius;
    float m_fShadowStrength;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fPadding;
    int m_iProjCubeTextureIndex;
    int m_iProjCubeBlurredTextureIndex;
    PhotonFace m_axFace[6];
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 4, std140) uniform type_PhotonIndex
{
    uint g_uPhotonIndex;
} PhotonIndex;

layout(set = 1, binding = 89, std430) readonly buffer type_StructuredBuffer_PhotonParams
{
    layout(row_major) PhotonParams _m0[];
} g_axPhotons;

layout(set = 0, binding = 3) uniform sampler g_xBilinearClamp;
layout(set = 0, binding = 8) uniform samplerShadow g_xShadowCmpSampler;
layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xLighting_AlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xLighting_PBRPropertiesBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;
layout(set = 1, binding = 56) uniform texture2D g_xShadowCubeMapPosX;
layout(set = 1, binding = 57) uniform texture2D g_xShadowCubeMapNegX;
layout(set = 1, binding = 59) uniform texture2D g_xShadowCubeMapNegY;
layout(set = 1, binding = 58) uniform texture2D g_xShadowCubeMapPosY;
layout(set = 1, binding = 60) uniform texture2D g_xShadowCubeMapPosZ;
layout(set = 1, binding = 61) uniform texture2D g_xShadowCubeMapNegZ;

layout(location = 0) out vec4 out_var_SV_Target;

vec3 _200;
vec2 _201;
vec2 _202;

void main()
{
    ivec2 _211 = ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy;
    float _219 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _211, 0).x);
    //vec3 _238 = (vec4((FrameConstants.g_xCameraTanHalfFOV * _219) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy)
    // - vec2(1.0)), _219, 1.0) * FrameConstants.g_xInvView).xyz;
	 vec4 temp = vec4((FrameConstants.g_xCameraTanHalfFOV * _219) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy)
     - vec2(1.0)), _219, 1.0);
 
     temp.x -= vk3d_params[VIEW].stereo.x * (_219 - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0];
     
    vec3 _238 = (temp * FrameConstants.g_xInvView).xyz;


    vec4 _240 = TEXEL_FETCH(g_xLighting_NormalBuffer, _211, 0);
    vec4 _242 = TEXEL_FETCH(g_xLighting_AlbedoBuffer, _211, 0);
    vec4 _244 = TEXEL_FETCH(g_xLighting_PBRPropertiesBuffer, _211, 0);
    vec3 _248 = normalize((_240.xyz * 2.0) - vec3(1.0));
    vec3 _249 = _242.xyz;
    float _250 = _244.y;
    float _251 = _242.w;
    float _252 = _244.x;
    vec2 _255 = (_244.zw * 2.0) - vec2(1.0);
    vec2 _257 = _255.xy;
    float _261 = sqrt(clamp(1.0 - dot(_257, _257), 0.0, 1.0));
    vec3 _266 = vec3(_255.x, _255.y, _200.z);
    _266.z = (_240.w != 0.0) ? _261 : (-_261);
    vec4 _2142;
    bool _2143;
    do
    {
        vec3 _308 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightPos - _238;
        float _309 = length(_308);
        vec3 _311 = _308 / vec3(_309);
        float _319 = pow(clamp((1.0 - clamp((_309 - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius) / (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRange - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius), 0.0, 1.0)) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightFalloffScale, 0.0, 1.0), g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightFalloffPower);
        float _327;
        if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_bUseLimitAngles != 0u)
        {
            _327 = _319 * clamp((dot(_311, g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightDir) + g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightAngleOuter) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fRcpLightAngleRange, 0.0, 1.0);
        }
        else
        {
            _327 = _319;
        }
        if (_327 < 9.9999997473787516355514526367188e-05)
        {
            _2142 = vec4(0.0, 0.0, 0.0, 1.0);
            _2143 = false;
            break;
        }
        float _332 = clamp(dot(_248, _311), 0.0, 1.0);
        vec3 _336 = -_308;
        vec3 _339 = _336;
        _339.y = -_336.y;
        vec3 _340 = abs(_339);
        float _341 = _340.x;
        float _342 = _340.y;
        float _343 = _340.z;
        float _345 = max(_341, max(_342, _343));
        vec3 _347 = _339 / vec3(_345);
        float _352 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xDepthProjection.x + (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xDepthProjection.y / (_345 - max(g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowBias * (1.0 - _332), g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowBias)));
        float _357 = (2.0 * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowFarPlane;
        float _358 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowFarPlane + g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane;
        float _359 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowFarPlane - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane;
        float _365 = clamp(((_357 / (_358 - ((((1.0 - _352) * 2.0) - 1.0) * _359))) - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane) / _359, 0.0, 1.0);
        vec2 _452;
        uint _453;
        if (_342 > _341)
        {
            vec2 _450;
            uint _451;
            if (_343 > _342)
            {
                bool _433 = _347.z < 0.0;
                vec2 _448;
                if (_433)
                {
                    vec2 _444 = _201;
                    _444.x = -_347.x;
                    vec2 _447 = _444;
                    _447.y = -_347.y;
                    _448 = _447;
                }
                else
                {
                    vec2 _438 = _201;
                    _438.x = _347.x;
                    vec2 _441 = _438;
                    _441.y = -_347.y;
                    _448 = _441;
                }
                _450 = _448;
                _451 = _433 ? 5u : 4u;
            }
            else
            {
                bool _417 = _347.y < 0.0;
                vec2 _430;
                if (_417)
                {
                    vec2 _426 = _201;
                    _426.x = _347.x;
                    vec2 _429 = _426;
                    _429.y = -_347.z;
                    _430 = _429;
                }
                else
                {
                    vec2 _422 = _201;
                    _422.x = _347.x;
                    vec2 _424 = _422;
                    _424.y = _347.z;
                    _430 = _424;
                }
                _450 = _430;
                _451 = _417 ? 2u : 3u;
            }
            _452 = _450;
            _453 = _451;
        }
        else
        {
            vec2 _410;
            uint _411;
            if (_343 > _341)
            {
                bool _393 = _347.z < 0.0;
                vec2 _408;
                if (_393)
                {
                    vec2 _404 = _201;
                    _404.x = -_347.x;
                    vec2 _407 = _404;
                    _407.y = -_347.y;
                    _408 = _407;
                }
                else
                {
                    vec2 _398 = _201;
                    _398.x = _347.x;
                    vec2 _401 = _398;
                    _401.y = -_347.y;
                    _408 = _401;
                }
                _410 = _408;
                _411 = _393 ? 5u : 4u;
            }
            else
            {
                bool _375 = _347.x < 0.0;
                vec2 _390;
                if (_375)
                {
                    vec2 _386 = _201;
                    _386.x = _347.z;
                    vec2 _389 = _386;
                    _389.y = -_347.y;
                    _390 = _389;
                }
                else
                {
                    vec2 _381 = _201;
                    _381.x = -_347.z;
                    vec2 _384 = _381;
                    _384.y = -_347.y;
                    _390 = _384;
                }
                _410 = _390;
                _411 = uint(_375);
            }
            _452 = _410;
            _453 = _411;
        }
        vec2 _455 = (_452 * 0.5) + vec2(0.5);
        vec4 _1870;
        if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowStrength > 0.0)
        {
            float _1866;
            do
            {
                if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].iShadowIndex == (-1))
                {
                    _1866 = 0.0;
                    break;
                }
                else
                {
                    float _1865;
                    if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowSoftness > 0.0)
                    {
                        bool _1084;
                        float _1065 = 6.28318500518798828125 * fract(52.98291778564453125 * fract(dot(gl_FragCoord.xy, vec2(0.067110560834407806396484375, 0.005837149918079376220703125))));
                        float _1066 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowSoftness * 0.02500000037252902984619140625;
                        float _1068 = float(uint(0));
                        float _1070 = sqrt(_1068 + 0.5);
                        float _1071 = _1070 * 0.5;
                        float _1073 = (_1068 * 2.400000095367431640625) + _1065;
                        float _1074 = sin(_1073);
                        float _1075 = cos(_1073);
                        float _1132;
                        do
                        {
                            vec2 _1083 = clamp(_455 + (vec2(_1071 * _1075, _1071 * _1074) * _1066), vec2(0.0), vec2(1.0));
                            _1084 = _453 == 0u;
                            if (_1084)
                            {
                                _1132 = texture(sampler2D(g_xShadowCubeMapPosX, g_xBilinearClamp), _1083).x;
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _1132 = texture(sampler2D(g_xShadowCubeMapNegX, g_xBilinearClamp), _1083).x;
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _1132 = texture(sampler2D(g_xShadowCubeMapPosY, g_xBilinearClamp), _1083).x;
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _1132 = texture(sampler2D(g_xShadowCubeMapNegY, g_xBilinearClamp), _1083).x;
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _1132 = texture(sampler2D(g_xShadowCubeMapPosZ, g_xBilinearClamp), _1083).x;
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _1132 = texture(sampler2D(g_xShadowCubeMapNegZ, g_xBilinearClamp), _1083).x;
                                break;
                            }
                            _1132 = 0.0;
                            break;
                        } while(false);
                        float _1141 = clamp(((_357 / (_358 - ((((1.0 - _1132) * 2.0) - 1.0) * _359))) - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane) / _359, 0.0, 1.0);
                        vec2 _1147;
                        if (_1141 < _365)
                        {
                            vec2 _1145 = _202;
                            _1145.x = _1141;
                            vec2 _1146 = _1145;
                            _1146.y = 1.0;
                            _1147 = _1146;
                        }
                        else
                        {
                            _1147 = vec2(0.0);
                        }
                        float _1149 = float(uint(1));
                        float _1151 = sqrt(_1149 + 0.5);
                        float _1152 = _1151 * 0.5;
                        float _1154 = (_1149 * 2.400000095367431640625) + _1065;
                        float _1155 = sin(_1154);
                        float _1156 = cos(_1154);
                        float _1212;
                        do
                        {
                            vec2 _1164 = clamp(_455 + (vec2(_1152 * _1156, _1152 * _1155) * _1066), vec2(0.0), vec2(1.0));
                            if (_1084)
                            {
                                _1212 = texture(sampler2D(g_xShadowCubeMapPosX, g_xBilinearClamp), _1164).x;
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _1212 = texture(sampler2D(g_xShadowCubeMapNegX, g_xBilinearClamp), _1164).x;
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _1212 = texture(sampler2D(g_xShadowCubeMapPosY, g_xBilinearClamp), _1164).x;
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _1212 = texture(sampler2D(g_xShadowCubeMapNegY, g_xBilinearClamp), _1164).x;
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _1212 = texture(sampler2D(g_xShadowCubeMapPosZ, g_xBilinearClamp), _1164).x;
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _1212 = texture(sampler2D(g_xShadowCubeMapNegZ, g_xBilinearClamp), _1164).x;
                                break;
                            }
                            _1212 = 0.0;
                            break;
                        } while(false);
                        float _1221 = clamp(((_357 / (_358 - ((((1.0 - _1212) * 2.0) - 1.0) * _359))) - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane) / _359, 0.0, 1.0);
                        vec2 _1231;
                        if (_1221 < _365)
                        {
                            vec2 _1227 = _202;
                            _1227.x = _1147.x + _1221;
                            vec2 _1230 = _1227;
                            _1230.y = _1147.y + 1.0;
                            _1231 = _1230;
                        }
                        else
                        {
                            _1231 = _1147;
                        }
                        float _1233 = float(uint(2));
                        float _1235 = sqrt(_1233 + 0.5);
                        float _1236 = _1235 * 0.5;
                        float _1238 = (_1233 * 2.400000095367431640625) + _1065;
                        float _1239 = sin(_1238);
                        float _1240 = cos(_1238);
                        float _1296;
                        do
                        {
                            vec2 _1248 = clamp(_455 + (vec2(_1236 * _1240, _1236 * _1239) * _1066), vec2(0.0), vec2(1.0));
                            if (_1084)
                            {
                                _1296 = texture(sampler2D(g_xShadowCubeMapPosX, g_xBilinearClamp), _1248).x;
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _1296 = texture(sampler2D(g_xShadowCubeMapNegX, g_xBilinearClamp), _1248).x;
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _1296 = texture(sampler2D(g_xShadowCubeMapPosY, g_xBilinearClamp), _1248).x;
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _1296 = texture(sampler2D(g_xShadowCubeMapNegY, g_xBilinearClamp), _1248).x;
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _1296 = texture(sampler2D(g_xShadowCubeMapPosZ, g_xBilinearClamp), _1248).x;
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _1296 = texture(sampler2D(g_xShadowCubeMapNegZ, g_xBilinearClamp), _1248).x;
                                break;
                            }
                            _1296 = 0.0;
                            break;
                        } while(false);
                        float _1305 = clamp(((_357 / (_358 - ((((1.0 - _1296) * 2.0) - 1.0) * _359))) - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane) / _359, 0.0, 1.0);
                        vec2 _1315;
                        if (_1305 < _365)
                        {
                            vec2 _1311 = _202;
                            _1311.x = _1231.x + _1305;
                            vec2 _1314 = _1311;
                            _1314.y = _1231.y + 1.0;
                            _1315 = _1314;
                        }
                        else
                        {
                            _1315 = _1231;
                        }
                        float _1317 = float(uint(3));
                        float _1319 = sqrt(_1317 + 0.5);
                        float _1320 = _1319 * 0.5;
                        float _1322 = (_1317 * 2.400000095367431640625) + _1065;
                        float _1323 = sin(_1322);
                        float _1324 = cos(_1322);
                        float _1380;
                        do
                        {
                            vec2 _1332 = clamp(_455 + (vec2(_1320 * _1324, _1320 * _1323) * _1066), vec2(0.0), vec2(1.0));
                            if (_1084)
                            {
                                _1380 = texture(sampler2D(g_xShadowCubeMapPosX, g_xBilinearClamp), _1332).x;
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _1380 = texture(sampler2D(g_xShadowCubeMapNegX, g_xBilinearClamp), _1332).x;
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _1380 = texture(sampler2D(g_xShadowCubeMapPosY, g_xBilinearClamp), _1332).x;
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _1380 = texture(sampler2D(g_xShadowCubeMapNegY, g_xBilinearClamp), _1332).x;
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _1380 = texture(sampler2D(g_xShadowCubeMapPosZ, g_xBilinearClamp), _1332).x;
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _1380 = texture(sampler2D(g_xShadowCubeMapNegZ, g_xBilinearClamp), _1332).x;
                                break;
                            }
                            _1380 = 0.0;
                            break;
                        } while(false);
                        float _1389 = clamp(((_357 / (_358 - ((((1.0 - _1380) * 2.0) - 1.0) * _359))) - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane) / _359, 0.0, 1.0);
                        vec2 _1399;
                        if (_1389 < _365)
                        {
                            vec2 _1395 = _202;
                            _1395.x = _1315.x + _1389;
                            vec2 _1398 = _1395;
                            _1398.y = _1315.y + 1.0;
                            _1399 = _1398;
                        }
                        else
                        {
                            _1399 = _1315;
                        }
                        float _1864;
                        if (_1399.y > 0.0)
                        {
                            float _1405 = _1399.x / _1399.y;
                            float _1407 = (_365 - _1405) / _1405;
                            float _1415 = ((clamp(_1407 * _1407, 0.0, 1.0) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowSoftness) * 0.02500000037252902984619140625) + (2.0 * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize);
                            float _1416 = _1070 * 0.3535533845424652099609375;
                            float _1466;
                            do
                            {
                                vec2 _1424 = clamp(_455 + (vec2(_1416 * _1075, _1416 * _1074) * _1415), vec2(0.0), vec2(1.0));
                                if (_1084)
                                {
                                    _1466 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1424, _352));
                                    break;
                                }
                                if (_453 == 1u)
                                {
                                    _1466 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1424, _352));
                                    break;
                                }
                                if (_453 == 2u)
                                {
                                    _1466 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1424, _352));
                                    break;
                                }
                                if (_453 == 3u)
                                {
                                    _1466 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1424, _352));
                                    break;
                                }
                                if (_453 == 4u)
                                {
                                    _1466 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1424, _352));
                                    break;
                                }
                                if (_453 == 5u)
                                {
                                    _1466 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1424, _352));
                                    break;
                                }
                                _1466 = 0.0;
                                break;
                            } while(false);
                            float _1467 = _1151 * 0.3535533845424652099609375;
                            float _1517;
                            do
                            {
                                vec2 _1475 = clamp(_455 + (vec2(_1467 * _1156, _1467 * _1155) * _1415), vec2(0.0), vec2(1.0));
                                if (_1084)
                                {
                                    _1517 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1475, _352));
                                    break;
                                }
                                if (_453 == 1u)
                                {
                                    _1517 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1475, _352));
                                    break;
                                }
                                if (_453 == 2u)
                                {
                                    _1517 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1475, _352));
                                    break;
                                }
                                if (_453 == 3u)
                                {
                                    _1517 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1475, _352));
                                    break;
                                }
                                if (_453 == 4u)
                                {
                                    _1517 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1475, _352));
                                    break;
                                }
                                if (_453 == 5u)
                                {
                                    _1517 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1475, _352));
                                    break;
                                }
                                _1517 = 0.0;
                                break;
                            } while(false);
                            float _1519 = _1235 * 0.3535533845424652099609375;
                            float _1569;
                            do
                            {
                                vec2 _1527 = clamp(_455 + (vec2(_1519 * _1240, _1519 * _1239) * _1415), vec2(0.0), vec2(1.0));
                                if (_1084)
                                {
                                    _1569 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1527, _352));
                                    break;
                                }
                                if (_453 == 1u)
                                {
                                    _1569 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1527, _352));
                                    break;
                                }
                                if (_453 == 2u)
                                {
                                    _1569 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1527, _352));
                                    break;
                                }
                                if (_453 == 3u)
                                {
                                    _1569 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1527, _352));
                                    break;
                                }
                                if (_453 == 4u)
                                {
                                    _1569 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1527, _352));
                                    break;
                                }
                                if (_453 == 5u)
                                {
                                    _1569 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1527, _352));
                                    break;
                                }
                                _1569 = 0.0;
                                break;
                            } while(false);
                            float _1571 = _1319 * 0.3535533845424652099609375;
                            float _1621;
                            do
                            {
                                vec2 _1579 = clamp(_455 + (vec2(_1571 * _1324, _1571 * _1323) * _1415), vec2(0.0), vec2(1.0));
                                if (_1084)
                                {
                                    _1621 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1579, _352));
                                    break;
                                }
                                if (_453 == 1u)
                                {
                                    _1621 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1579, _352));
                                    break;
                                }
                                if (_453 == 2u)
                                {
                                    _1621 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1579, _352));
                                    break;
                                }
                                if (_453 == 3u)
                                {
                                    _1621 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1579, _352));
                                    break;
                                }
                                if (_453 == 4u)
                                {
                                    _1621 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1579, _352));
                                    break;
                                }
                                if (_453 == 5u)
                                {
                                    _1621 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1579, _352));
                                    break;
                                }
                                _1621 = 0.0;
                                break;
                            } while(false);
                            float _1624 = float(uint(4));
                            float _1627 = sqrt(_1624 + 0.5) * 0.3535533845424652099609375;
                            float _1629 = (_1624 * 2.400000095367431640625) + _1065;
                            float _1681;
                            do
                            {
                                vec2 _1639 = clamp(_455 + (vec2(_1627 * cos(_1629), _1627 * sin(_1629)) * _1415), vec2(0.0), vec2(1.0));
                                if (_1084)
                                {
                                    _1681 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1639, _352));
                                    break;
                                }
                                if (_453 == 1u)
                                {
                                    _1681 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1639, _352));
                                    break;
                                }
                                if (_453 == 2u)
                                {
                                    _1681 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1639, _352));
                                    break;
                                }
                                if (_453 == 3u)
                                {
                                    _1681 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1639, _352));
                                    break;
                                }
                                if (_453 == 4u)
                                {
                                    _1681 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1639, _352));
                                    break;
                                }
                                if (_453 == 5u)
                                {
                                    _1681 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1639, _352));
                                    break;
                                }
                                _1681 = 0.0;
                                break;
                            } while(false);
                            float _1684 = float(uint(5));
                            float _1687 = sqrt(_1684 + 0.5) * 0.3535533845424652099609375;
                            float _1689 = (_1684 * 2.400000095367431640625) + _1065;
                            float _1741;
                            do
                            {
                                vec2 _1699 = clamp(_455 + (vec2(_1687 * cos(_1689), _1687 * sin(_1689)) * _1415), vec2(0.0), vec2(1.0));
                                if (_1084)
                                {
                                    _1741 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1699, _352));
                                    break;
                                }
                                if (_453 == 1u)
                                {
                                    _1741 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1699, _352));
                                    break;
                                }
                                if (_453 == 2u)
                                {
                                    _1741 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1699, _352));
                                    break;
                                }
                                if (_453 == 3u)
                                {
                                    _1741 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1699, _352));
                                    break;
                                }
                                if (_453 == 4u)
                                {
                                    _1741 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1699, _352));
                                    break;
                                }
                                if (_453 == 5u)
                                {
                                    _1741 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1699, _352));
                                    break;
                                }
                                _1741 = 0.0;
                                break;
                            } while(false);
                            float _1744 = float(uint(6));
                            float _1747 = sqrt(_1744 + 0.5) * 0.3535533845424652099609375;
                            float _1749 = (_1744 * 2.400000095367431640625) + _1065;
                            float _1801;
                            do
                            {
                                vec2 _1759 = clamp(_455 + (vec2(_1747 * cos(_1749), _1747 * sin(_1749)) * _1415), vec2(0.0), vec2(1.0));
                                if (_1084)
                                {
                                    _1801 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1759, _352));
                                    break;
                                }
                                if (_453 == 1u)
                                {
                                    _1801 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1759, _352));
                                    break;
                                }
                                if (_453 == 2u)
                                {
                                    _1801 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1759, _352));
                                    break;
                                }
                                if (_453 == 3u)
                                {
                                    _1801 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1759, _352));
                                    break;
                                }
                                if (_453 == 4u)
                                {
                                    _1801 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1759, _352));
                                    break;
                                }
                                if (_453 == 5u)
                                {
                                    _1801 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1759, _352));
                                    break;
                                }
                                _1801 = 0.0;
                                break;
                            } while(false);
                            float _1804 = float(uint(7));
                            float _1807 = sqrt(_1804 + 0.5) * 0.3535533845424652099609375;
                            float _1809 = (_1804 * 2.400000095367431640625) + _1065;
                            float _1861;
                            do
                            {
                                vec2 _1819 = clamp(_455 + (vec2(_1807 * cos(_1809), _1807 * sin(_1809)) * _1415), vec2(0.0), vec2(1.0));
                                if (_1084)
                                {
                                    _1861 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1819, _352));
                                    break;
                                }
                                if (_453 == 1u)
                                {
                                    _1861 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1819, _352));
                                    break;
                                }
                                if (_453 == 2u)
                                {
                                    _1861 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1819, _352));
                                    break;
                                }
                                if (_453 == 3u)
                                {
                                    _1861 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1819, _352));
                                    break;
                                }
                                if (_453 == 4u)
                                {
                                    _1861 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1819, _352));
                                    break;
                                }
                                if (_453 == 5u)
                                {
                                    _1861 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1819, _352));
                                    break;
                                }
                                _1861 = 0.0;
                                break;
                            } while(false);
                            _1864 = (((((((_1466 + _1517) + _1569) + _1621) + _1681) + _1741) + _1801) + _1861) * 0.125;
                        }
                        else
                        {
                            _1864 = 0.0;
                        }
                        _1865 = _1864;
                    }
                    else
                    {
                        bool _478;
                        float _520;
                        do
                        {
                            vec2 _477 = clamp(_455 + (vec2(-0.326211988925933837890625, -0.40580499172210693359375) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize), vec2(0.0), vec2(1.0));
                            _478 = _453 == 0u;
                            if (_478)
                            {
                                _520 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_477, _352));
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _520 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_477, _352));
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _520 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_477, _352));
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _520 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_477, _352));
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _520 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_477, _352));
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _520 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_477, _352));
                                break;
                            }
                            _520 = 0.0;
                            break;
                        } while(false);
                        float _568;
                        do
                        {
                            vec2 _526 = clamp(_455 + (vec2(-0.840143978595733642578125, -0.0735799968242645263671875) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize), vec2(0.0), vec2(1.0));
                            if (_478)
                            {
                                _568 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_526, _352));
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _568 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_526, _352));
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _568 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_526, _352));
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _568 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_526, _352));
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _568 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_526, _352));
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _568 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_526, _352));
                                break;
                            }
                            _568 = 0.0;
                            break;
                        } while(false);
                        float _617;
                        do
                        {
                            vec2 _575 = clamp(_455 + (vec2(-0.695913970470428466796875, 0.45713698863983154296875) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize), vec2(0.0), vec2(1.0));
                            if (_478)
                            {
                                _617 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_575, _352));
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _617 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_575, _352));
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _617 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_575, _352));
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _617 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_575, _352));
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _617 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_575, _352));
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _617 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_575, _352));
                                break;
                            }
                            _617 = 0.0;
                            break;
                        } while(false);
                        float _666;
                        do
                        {
                            vec2 _624 = clamp(_455 + (vec2(-0.203345000743865966796875, 0.62071597576141357421875) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize), vec2(0.0), vec2(1.0));
                            if (_478)
                            {
                                _666 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_624, _352));
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _666 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_624, _352));
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _666 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_624, _352));
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _666 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_624, _352));
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _666 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_624, _352));
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _666 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_624, _352));
                                break;
                            }
                            _666 = 0.0;
                            break;
                        } while(false);
                        float _715;
                        do
                        {
                            vec2 _673 = clamp(_455 + (vec2(0.96233999729156494140625, -0.194983005523681640625) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize), vec2(0.0), vec2(1.0));
                            if (_478)
                            {
                                _715 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_673, _352));
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _715 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_673, _352));
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _715 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_673, _352));
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _715 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_673, _352));
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _715 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_673, _352));
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _715 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_673, _352));
                                break;
                            }
                            _715 = 0.0;
                            break;
                        } while(false);
                        float _764;
                        do
                        {
                            vec2 _722 = clamp(_455 + (vec2(0.4734340012073516845703125, -0.4800260066986083984375) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize), vec2(0.0), vec2(1.0));
                            if (_478)
                            {
                                _764 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_722, _352));
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _764 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_722, _352));
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _764 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_722, _352));
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _764 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_722, _352));
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _764 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_722, _352));
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _764 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_722, _352));
                                break;
                            }
                            _764 = 0.0;
                            break;
                        } while(false);
                        float _813;
                        do
                        {
                            vec2 _771 = clamp(_455 + (vec2(0.51945602893829345703125, 0.76702201366424560546875) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize), vec2(0.0), vec2(1.0));
                            if (_478)
                            {
                                _813 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_771, _352));
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _813 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_771, _352));
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _813 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_771, _352));
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _813 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_771, _352));
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _813 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_771, _352));
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _813 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_771, _352));
                                break;
                            }
                            _813 = 0.0;
                            break;
                        } while(false);
                        float _862;
                        do
                        {
                            vec2 _820 = clamp(_455 + (vec2(0.18546099960803985595703125, -0.89312398433685302734375) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize), vec2(0.0), vec2(1.0));
                            if (_478)
                            {
                                _862 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_820, _352));
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _862 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_820, _352));
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _862 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_820, _352));
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _862 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_820, _352));
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _862 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_820, _352));
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _862 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_820, _352));
                                break;
                            }
                            _862 = 0.0;
                            break;
                        } while(false);
                        float _911;
                        do
                        {
                            vec2 _869 = clamp(_455 + (vec2(0.507430970668792724609375, 0.064424999058246612548828125) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize), vec2(0.0), vec2(1.0));
                            if (_478)
                            {
                                _911 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_869, _352));
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _911 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_869, _352));
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _911 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_869, _352));
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _911 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_869, _352));
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _911 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_869, _352));
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _911 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_869, _352));
                                break;
                            }
                            _911 = 0.0;
                            break;
                        } while(false);
                        float _960;
                        do
                        {
                            vec2 _918 = clamp(_455 + (vec2(0.896420001983642578125, 0.412458002567291259765625) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize), vec2(0.0), vec2(1.0));
                            if (_478)
                            {
                                _960 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_918, _352));
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _960 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_918, _352));
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _960 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_918, _352));
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _960 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_918, _352));
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _960 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_918, _352));
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _960 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_918, _352));
                                break;
                            }
                            _960 = 0.0;
                            break;
                        } while(false);
                        float _1009;
                        do
                        {
                            vec2 _967 = clamp(_455 + (vec2(-0.321940004825592041015625, -0.932614982128143310546875) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize), vec2(0.0), vec2(1.0));
                            if (_478)
                            {
                                _1009 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_967, _352));
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _1009 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_967, _352));
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _1009 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_967, _352));
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _1009 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_967, _352));
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _1009 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_967, _352));
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _1009 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_967, _352));
                                break;
                            }
                            _1009 = 0.0;
                            break;
                        } while(false);
                        float _1058;
                        do
                        {
                            vec2 _1016 = clamp(_455 + (vec2(-0.7915589809417724609375, -0.59770500659942626953125) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_453].fInvSize), vec2(0.0), vec2(1.0));
                            if (_478)
                            {
                                _1058 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1016, _352));
                                break;
                            }
                            if (_453 == 1u)
                            {
                                _1058 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1016, _352));
                                break;
                            }
                            if (_453 == 2u)
                            {
                                _1058 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1016, _352));
                                break;
                            }
                            if (_453 == 3u)
                            {
                                _1058 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1016, _352));
                                break;
                            }
                            if (_453 == 4u)
                            {
                                _1058 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1016, _352));
                                break;
                            }
                            if (_453 == 5u)
                            {
                                _1058 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1016, _352));
                                break;
                            }
                            _1058 = 0.0;
                            break;
                        } while(false);
                        _1865 = (((((((((((_520 + _568) + _617) + _666) + _715) + _764) + _813) + _862) + _911) + _960) + _1009) + _1058) * 0.083333335816860198974609375;
                    }
                    _1866 = _1865;
                    break;
                }
            } while(false);
            vec4 _1869 = vec4(0.0, 0.0, 0.0, 1.0);
            _1869.w = 1.0 - (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowStrength * _1866);
            _1870 = _1869;
        }
        else
        {
            _1870 = vec4(0.0, 0.0, 0.0, 1.0);
        }
        if (_1870.w > 0.0)
        {
            vec3 _1875 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightColour * _327;
            vec3 _1877 = normalize(FrameConstants.g_xCamPos - _238);
            float _1895;
            vec3 _1896;
            if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius > 0.0)
            {
                vec3 _1882 = -reflect(_1877, _248);
                vec3 _1885 = (_1882 * dot(_308, _1882)) - _308;
                _1895 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius * inversesqrt(dot(_308, _308));
                _1896 = normalize(_308 + (_1885 * clamp(g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fAreaLightRadius / length(_1885), 0.0, 1.0)));
            }
            else
            {
                _1895 = 0.0;
                _1896 = _311;
            }
            float _1898 = (_251 != 0.01960784383118152618408203125) ? _252 : 0.0;
            vec3 _1900 = normalize(_1877 + _1896);
            float _1902 = clamp(dot(_248, _1877), 0.0, 1.0);
            float _1904 = clamp(dot(_248, _1900), 0.0, 1.0);
            float _1906 = clamp(dot(_1877, _1900), 0.0, 1.0);
            bool _1907 = _251 == 0.01568627543747425079345703125;
            vec3 _1944;
            vec3 _1945;
            vec3 _1946;
            if (_1907)
            {
                _1944 = vec3(0.046999998390674591064453125);
                _1945 = vec3(_332);
                _1946 = _249;
            }
            else
            {
                vec3 _1940;
                vec3 _1941;
                vec3 _1942;
                if (_251 == 0.007843137718737125396728515625)
                {
                    _1940 = vec3(0.0280000008642673492431640625);
                    _1941 = vec3(_332);
                    _1942 = _249;
                }
                else
                {
                    vec3 _1936;
                    vec3 _1937;
                    vec3 _1938;
                    if (_251 == 0.0039215688593685626983642578125)
                    {
                        _1936 = vec3(0.0280000008642673492431640625);
                        _1937 = vec3((clamp((clamp(dot(_311, -_248), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_311, -_1877), 0.0, 1.0), 7.0), 0.0, 1.0) * _1898) + _332);
                        _1938 = _249;
                    }
                    else
                    {
                        vec3 _1919 = vec3(_1898);
                        _1936 = mix(vec3(0.039999999105930328369140625), _249, _1919);
                        _1937 = vec3(_332);
                        _1938 = mix(_249, vec3(0.0), _1919);
                    }
                    _1940 = _1936;
                    _1941 = _1937;
                    _1942 = _1938;
                }
                _1944 = _1940;
                _1945 = _1941;
                _1946 = _1942;
            }
            vec3 _1947 = _1945 * _1946;
            float _1948 = max(_250, 0.0500000007450580596923828125);
            float _1949 = _1948 * _1948;
            vec3 _2062;
            if (_1907)
            {
                float _2000 = clamp(1.0 - _250, 0.0, 1.0);
                float _2002 = ((_252 * 2.0) - 1.0) * 0.300000011920928955078125;
                float _2008 = pow(_2000, 8.0) * 500.0;
                float _2011 = pow(_2000, 6.0) * 2000.0;
                float _2025 = dot(normalize(_266 + (_248 * ((-0.0500000007450580596923828125) + _2002))), _1900);
                float _2045 = dot(normalize(_266 + (_248 * ((0.0500000007450580596923828125 + _2002) - (_2000 * 0.0500000007450580596923828125)))), _1900);
                _2062 = _1947 + ((((vec3(((smoothstep(-1.0, 0.0, _2025) * pow(sqrt(clamp(1.0 - (_2025 * _2025), 0.0, 1.0)), 20.0 + _2008)) * ((((_252 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _2000))) * ((_2008 + 28.0) * 0.03978873789310455322265625)) * (_249 / vec3(max(dot(_249, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _2045) * pow(sqrt(clamp(1.0 - (_2045 * _2045), 0.0, 1.0)), 50.0 + _2011)) * ((((_252 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _2000)) * ((_2011 + 58.0) * 0.03978873789310455322265625))) * _332) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fSpecularStrength);
            }
            else
            {
                float _1955 = clamp(_1895 * (1.0 - _1949), 0.0, 1.0);
                float _1975;
                if (_1955 > 0.0)
                {
                    float _1966 = _1949 * _1949;
                    _1975 = _1966 / (_1966 + (((_1955 * 0.25) * ((3.0 * _1949) + _1955)) / (_1906 + 0.001000000047497451305389404296875)));
                }
                else
                {
                    _1975 = 1.0;
                }
                float _1976 = 2.0 - _1949;
                float _1983 = _1949 * _1949;
                float _1987 = (((_1904 * _1983) - _1904) * _1904) + 1.0;
                _2062 = _1947 + (((((vec3(1.0) - _1944) * exp2((((-5.554729938507080078125) * _1906) - 6.9831600189208984375) * _1906)) + _1944) * ((((1.0 / (((_1902 * _1976) + _1949) * ((_332 * _1976) + _1949))) * (_1983 * (1.0 / (_1987 * _1987)))) * _332) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fSpecularStrength)) * _1975);
            }
            vec3 _2112;
            if (_251 == 0.01960784383118152618408203125)
            {
                bool _2066 = _252 > 0.4980392158031463623046875;
                vec3 _2084 = sqrt(abs(_1946));
                float _2098 = 0.5 / _1949;
                _2112 = mix(_2062, (_1947 + (((((vec3(1.0) - _2084) * exp2((((-5.554729938507080078125) * _1906) - 6.9831600189208984375) * _1906)) + _2084) * ((1.0 / ((4.0 * ((_332 + _1902) - (_332 * _1902))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _2098) * pow(1.0 - (_1904 * _1904), _2098)))) * (_332 * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fSpecularStrength))) + (_1946 * (clamp((clamp(dot(_311, -_248), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_311, -_1877), 0.0, 1.0), 7.0), 0.0, 1.0) * (_2066 ? 0.5 : 0.0))), vec3((_2066 ? (_252 - 0.501960813999176025390625) : _252) * 2.007874011993408203125));
            }
            else
            {
                _2112 = _2062;
            }
            vec3 _2115 = min(_2112 * _1875, vec3(1000.0)).xyz;
            bool _2116 = _251 == 0.007843137718737125396728515625;
            vec3 _2127;
            if (_2116)
            {
                _2127 = _2115 + (((_249 * vec3(0.0194447897374629974365234375, 1.6805228369776159524917602539062e-05, 3.1907067266079280898338765837252e-11)) * clamp(0.300000011920928955078125 + dot(_311, -_248), 0.0, 1.0)) * _1875);
            }
            else
            {
                _2127 = _2115;
            }
            vec3 _2140;
            if (_2116 && (_219 < 3.0))
            {
                _2140 = _2127.xyz * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(_219 - 2.0, 0.0)));
            }
            else
            {
                _2140 = _2127.xyz;
            }
            _2142 = vec4(_2140.x, _2140.y, _2140.z, _1870.w);
            _2143 = true;
            break;
        }
        _2142 = _1870;
        _2143 = false;
        break;
    } while(false);
    if (false == _2143)
    {
        discard;
    }
    out_var_SV_Target = _2142;
}


