// LIGHTS 1

#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct PhotonFace
{
    uint uLevel;
    float fSize;
    float fInvSize;
    int iShadowIndex;
};

struct PhotonParams
{
    mat4 m_xLightOrientation;
    mat4 m_xCustomVolumeMatrix;
    vec3 m_xLightPos;
    float m_fLightRadius;
    vec3 m_xLightDir;
    float m_fLightRange;
    vec3 m_xLightColour;
    uint m_bUseLimitAngles;
    float m_fLightFalloffScale;
    float m_fLightFalloffPower;
    float m_fLightAngleOuter;
    float m_fRcpLightAngleRange;
    float m_fArcCosLimitAngle;
    float m_fRcpProjCubeLerpDist;
    uint m_bUseProjCube;
    uint m_bUseDualProjCube;
    vec2 m_xDepthProjection;
    float m_fLightMeshRadius;
    float m_fShadowStrength;
    float m_fShadowSoftness;
    float m_fShadowNearPlane;
    float m_fShadowFarPlane;
    float m_fShadowBias;
    float m_fSpecularStrength;
    float m_fPadding;
    int m_iProjCubeTextureIndex;
    int m_iProjCubeBlurredTextureIndex;
    PhotonFace m_axFace[6];
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 4, std140) uniform type_PhotonIndex
{
    uint g_uPhotonIndex;
} PhotonIndex;

layout(set = 1, binding = 89, std430) readonly buffer type_StructuredBuffer_PhotonParams
{
    layout(row_major) PhotonParams _m0[];
} g_axPhotons;

layout(set = 0, binding = 3) uniform sampler g_xBilinearClamp;
layout(set = 0, binding = 5) uniform sampler g_xTrilinearClamp;
layout(set = 0, binding = 8) uniform samplerShadow g_xShadowCmpSampler;
layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xLighting_AlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xLighting_PBRPropertiesBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;
layout(set = 1, binding = 56) uniform texture2D g_xShadowCubeMapPosX;
layout(set = 1, binding = 57) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_xShadowCubeMapNegX;
layout(set = 1, binding = 59) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_xShadowCubeMapNegY;
layout(set = 1, binding = 58) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_xShadowCubeMapPosY;
layout(set = 1, binding = 60) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_xShadowCubeMapPosZ;
layout(set = 1, binding = 61) 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
uniform texture2D g_xShadowCubeMapNegZ;
layout(set = 1, binding = 54) uniform textureCube g_xProjCubeMap;
layout(set = 1, binding = 55) uniform textureCube g_xProjCubeMapBlurred;

layout(location = 0) out vec4 out_var_SV_Target;

vec3 _210;
vec2 _211;
vec2 _212;

void main()
{
    ivec2 _221 = ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy;
    float _229 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _221, 0).x);
    //vec3 _248 = (vec4((FrameConstants.g_xCameraTanHalfFOV * _229) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy)
     //- vec2(1.0)), _229, 1.0) * FrameConstants.g_xInvView).xyz;
    
    vec4 temp = vec4((FrameConstants.g_xCameraTanHalfFOV * _229) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy)
     - vec2(1.0)), _229, 1.0);
     
     temp.x -= vk3d_params[VIEW].stereo.x * (_229 - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0];
     
    vec3 _248 = (temp * FrameConstants.g_xInvView).xyz;
     
     
    vec4 _250 = TEXEL_FETCH(g_xLighting_NormalBuffer, _221, 0);
    vec4 _252 = TEXEL_FETCH(g_xLighting_AlbedoBuffer, _221, 0);
    vec4 _254 = TEXEL_FETCH(g_xLighting_PBRPropertiesBuffer, _221, 0);
    vec3 _258 = normalize((_250.xyz * 2.0) - vec3(1.0));
    vec3 _259 = _252.xyz;
    float _260 = _254.y;
    float _261 = _252.w;
    float _262 = _254.x;
    vec2 _265 = (_254.zw * 2.0) - vec2(1.0);
    vec2 _267 = _265.xy;
    float _271 = sqrt(clamp(1.0 - dot(_267, _267), 0.0, 1.0));
    vec3 _276 = vec3(_265.x, _265.y, _210.z);
    _276.z = (_250.w != 0.0) ? _271 : (-_271);
    vec4 _2191;
    bool _2192;
    do
    {
        vec3 _330 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightPos - _248;
        float _331 = length(_330);
        vec3 _333 = _330 / vec3(_331);
        float _341 = pow(clamp((1.0 - clamp((_331 - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRadius) / (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRange - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRadius), 0.0, 1.0)) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightFalloffScale, 0.0, 1.0), g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightFalloffPower);
        float _349;
        if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_bUseLimitAngles != 0u)
        {
            _349 = _341 * clamp((dot(_333, g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightDir) + g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightAngleOuter) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fRcpLightAngleRange, 0.0, 1.0);
        }
        else
        {
            _349 = _341;
        }
        if (_349 < 9.9999997473787516355514526367188e-05)
        {
            _2191 = vec4(0.0, 0.0, 0.0, 1.0);
            _2192 = false;
            break;
        }
        float _354 = clamp(dot(_258, _333), 0.0, 1.0);
        vec3 _358 = -_330;
        vec3 _361 = _358;
        _361.y = -_358.y;
        vec3 _362 = abs(_361);
        float _363 = _362.x;
        float _364 = _362.y;
        float _365 = _362.z;
        float _367 = max(_363, max(_364, _365));
        vec3 _369 = _361 / vec3(_367);
        float _374 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xDepthProjection.x + (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xDepthProjection.y / (_367 - max(g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowBias * (1.0 - _354), g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowBias)));
        float _379 = (2.0 * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowFarPlane;
        float _380 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowFarPlane + g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane;
        float _381 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowFarPlane - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane;
        float _387 = clamp(((_379 / (_380 - ((((1.0 - _374) * 2.0) - 1.0) * _381))) - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane) / _381, 0.0, 1.0);
        vec2 _474;
        uint _475;
        if (_364 > _363)
        {
            vec2 _472;
            uint _473;
            if (_365 > _364)
            {
                bool _455 = _369.z < 0.0;
                vec2 _470;
                if (_455)
                {
                    vec2 _466 = _211;
                    _466.x = -_369.x;
                    vec2 _469 = _466;
                    _469.y = -_369.y;
                    _470 = _469;
                }
                else
                {
                    vec2 _460 = _211;
                    _460.x = _369.x;
                    vec2 _463 = _460;
                    _463.y = -_369.y;
                    _470 = _463;
                }
                _472 = _470;
                _473 = _455 ? 5u : 4u;
            }
            else
            {
                bool _439 = _369.y < 0.0;
                vec2 _452;
                if (_439)
                {
                    vec2 _448 = _211;
                    _448.x = _369.x;
                    vec2 _451 = _448;
                    _451.y = -_369.z;
                    _452 = _451;
                }
                else
                {
                    vec2 _444 = _211;
                    _444.x = _369.x;
                    vec2 _446 = _444;
                    _446.y = _369.z;
                    _452 = _446;
                }
                _472 = _452;
                _473 = _439 ? 2u : 3u;
            }
            _474 = _472;
            _475 = _473;
        }
        else
        {
            vec2 _432;
            uint _433;
            if (_365 > _363)
            {
                bool _415 = _369.z < 0.0;
                vec2 _430;
                if (_415)
                {
                    vec2 _426 = _211;
                    _426.x = -_369.x;
                    vec2 _429 = _426;
                    _429.y = -_369.y;
                    _430 = _429;
                }
                else
                {
                    vec2 _420 = _211;
                    _420.x = _369.x;
                    vec2 _423 = _420;
                    _423.y = -_369.y;
                    _430 = _423;
                }
                _432 = _430;
                _433 = _415 ? 5u : 4u;
            }
            else
            {
                bool _397 = _369.x < 0.0;
                vec2 _412;
                if (_397)
                {
                    vec2 _408 = _211;
                    _408.x = _369.z;
                    vec2 _411 = _408;
                    _411.y = -_369.y;
                    _412 = _411;
                }
                else
                {
                    vec2 _403 = _211;
                    _403.x = -_369.z;
                    vec2 _406 = _403;
                    _406.y = -_369.y;
                    _412 = _406;
                }
                _432 = _412;
                _433 = uint(_397);
            }
            _474 = _432;
            _475 = _433;
        }
        vec2 _477 = (_474 * 0.5) + vec2(0.5);
        vec4 _1892;
        if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowStrength > 0.0)
        {
            float _1888;
            do
            {
                if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].iShadowIndex == (-1))
                {
                    _1888 = 0.0;
                    break;
                }
                else
                {
                    float _1887;
                    if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowSoftness > 0.0)
                    {
                        bool _1106;
                        float _1087 = 6.28318500518798828125 * fract(52.98291778564453125 * fract(dot(gl_FragCoord.xy, vec2(0.067110560834407806396484375, 0.005837149918079376220703125))));
                        float _1088 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowSoftness * 0.02500000037252902984619140625;
                        float _1090 = float(uint(0));
                        float _1092 = sqrt(_1090 + 0.5);
                        float _1093 = _1092 * 0.5;
                        float _1095 = (_1090 * 2.400000095367431640625) + _1087;
                        float _1096 = sin(_1095);
                        float _1097 = cos(_1095);
                        float _1154;
                        do
                        {
                            vec2 _1105 = clamp(_477 + (vec2(_1093 * _1097, _1093 * _1096) * _1088), vec2(0.0), vec2(1.0));
                            _1106 = _475 == 0u;
                            if (_1106)
                            {
                                _1154 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapPosX, g_xBilinearClamp), _1105).x;
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _1154 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapNegX, g_xBilinearClamp), _1105).x;
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _1154 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapPosY, g_xBilinearClamp), _1105).x;
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _1154 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapNegY, g_xBilinearClamp), _1105).x;
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _1154 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapPosZ, g_xBilinearClamp), _1105).x;
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _1154 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapNegZ, g_xBilinearClamp), _1105).x;
                                break;
                            }
                            _1154 = 0.0;
                            break;
                        } while(false);
                        float _1163 = clamp(((_379 / (_380 - ((((1.0 - _1154) * 2.0) - 1.0) * _381))) - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane) / _381, 0.0, 1.0);
                        vec2 _1169;
                        if (_1163 < _387)
                        {
                            vec2 _1167 = _212;
                            _1167.x = _1163;
                            vec2 _1168 = _1167;
                            _1168.y = 1.0;
                            _1169 = _1168;
                        }
                        else
                        {
                            _1169 = vec2(0.0);
                        }
                        float _1171 = float(uint(1));
                        float _1173 = sqrt(_1171 + 0.5);
                        float _1174 = _1173 * 0.5;
                        float _1176 = (_1171 * 2.400000095367431640625) + _1087;
                        float _1177 = sin(_1176);
                        float _1178 = cos(_1176);
                        float _1234;
                        do
                        {
                            vec2 _1186 = clamp(_477 + (vec2(_1174 * _1178, _1174 * _1177) * _1088), vec2(0.0), vec2(1.0));
                            if (_1106)
                            {
                                _1234 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapPosX, g_xBilinearClamp), _1186).x;
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _1234 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapNegX, g_xBilinearClamp), _1186).x;
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _1234 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapPosY, g_xBilinearClamp), _1186).x;
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _1234 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapNegY, g_xBilinearClamp), _1186).x;
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _1234 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapPosZ, g_xBilinearClamp), _1186).x;
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _1234 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapNegZ, g_xBilinearClamp), _1186).x;
                                break;
                            }
                            _1234 = 0.0;
                            break;
                        } while(false);
                        float _1243 = clamp(((_379 / (_380 - ((((1.0 - _1234) * 2.0) - 1.0) * _381))) - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane) / _381, 0.0, 1.0);
                        vec2 _1253;
                        if (_1243 < _387)
                        {
                            vec2 _1249 = _212;
                            _1249.x = _1169.x + _1243;
                            vec2 _1252 = _1249;
                            _1252.y = _1169.y + 1.0;
                            _1253 = _1252;
                        }
                        else
                        {
                            _1253 = _1169;
                        }
                        float _1255 = float(uint(2));
                        float _1257 = sqrt(_1255 + 0.5);
                        float _1258 = _1257 * 0.5;
                        float _1260 = (_1255 * 2.400000095367431640625) + _1087;
                        float _1261 = sin(_1260);
                        float _1262 = cos(_1260);
                        float _1318;
                        do
                        {
                            vec2 _1270 = clamp(_477 + (vec2(_1258 * _1262, _1258 * _1261) * _1088), vec2(0.0), vec2(1.0));
                            if (_1106)
                            {
                                _1318 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapPosX, g_xBilinearClamp), _1270).x;
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _1318 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapNegX, g_xBilinearClamp), _1270).x;
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _1318 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapPosY, g_xBilinearClamp), _1270).x;
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _1318 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapNegY, g_xBilinearClamp), _1270).x;
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _1318 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapPosZ, g_xBilinearClamp), _1270).x;
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _1318 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapNegZ, g_xBilinearClamp), _1270).x;
                                break;
                            }
                            _1318 = 0.0;
                            break;
                        } while(false);
                        float _1327 = clamp(((_379 / (_380 - ((((1.0 - _1318) * 2.0) - 1.0) * _381))) - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane) / _381, 0.0, 1.0);
                        vec2 _1337;
                        if (_1327 < _387)
                        {
                            vec2 _1333 = _212;
                            _1333.x = _1253.x + _1327;
                            vec2 _1336 = _1333;
                            _1336.y = _1253.y + 1.0;
                            _1337 = _1336;
                        }
                        else
                        {
                            _1337 = _1253;
                        }
                        float _1339 = float(uint(3));
                        float _1341 = sqrt(_1339 + 0.5);
                        float _1342 = _1341 * 0.5;
                        float _1344 = (_1339 * 2.400000095367431640625) + _1087;
                        float _1345 = sin(_1344);
                        float _1346 = cos(_1344);
                        float _1402;
                        do
                        {
                            vec2 _1354 = clamp(_477 + (vec2(_1342 * _1346, _1342 * _1345) * _1088), vec2(0.0), vec2(1.0));
                            if (_1106)
                            {
                                _1402 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapPosX, g_xBilinearClamp), _1354).x;
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _1402 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapNegX, g_xBilinearClamp), _1354).x;
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _1402 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapPosY, g_xBilinearClamp), _1354).x;
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _1402 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapNegY, g_xBilinearClamp), _1354).x;
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _1402 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapPosZ, g_xBilinearClamp), _1354).x;
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _1402 = 
// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
texture(sampler2D(g_xShadowCubeMapNegZ, g_xBilinearClamp), _1354).x;
                                break;
                            }
                            _1402 = 0.0;
                            break;
                        } while(false);
                        float _1411 = clamp(((_379 / (_380 - ((((1.0 - _1402) * 2.0) - 1.0) * _381))) - g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowNearPlane) / _381, 0.0, 1.0);
                        vec2 _1421;
                        if (_1411 < _387)
                        {
                            vec2 _1417 = _212;
                            _1417.x = _1337.x + _1411;
                            vec2 _1420 = _1417;
                            _1420.y = _1337.y + 1.0;
                            _1421 = _1420;
                        }
                        else
                        {
                            _1421 = _1337;
                        }
                        float _1886;
                        if (_1421.y > 0.0)
                        {
                            float _1427 = _1421.x / _1421.y;
                            float _1429 = (_387 - _1427) / _1427;
                            float _1437 = ((clamp(_1429 * _1429, 0.0, 1.0) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowSoftness) * 0.02500000037252902984619140625) + (2.0 * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize);
                            float _1438 = _1092 * 0.3535533845424652099609375;
                            float _1488;
                            do
                            {
                                vec2 _1446 = clamp(_477 + (vec2(_1438 * _1097, _1438 * _1096) * _1437), vec2(0.0), vec2(1.0));
                                if (_1106)
                                {
                                    _1488 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1446, _374));
                                    break;
                                }
                                if (_475 == 1u)
                                {
                                    _1488 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1446, _374));
                                    break;
                                }
                                if (_475 == 2u)
                                {
                                    _1488 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1446, _374));
                                    break;
                                }
                                if (_475 == 3u)
                                {
                                    _1488 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1446, _374));
                                    break;
                                }
                                if (_475 == 4u)
                                {
                                    _1488 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1446, _374));
                                    break;
                                }
                                if (_475 == 5u)
                                {
                                    _1488 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1446, _374));
                                    break;
                                }
                                _1488 = 0.0;
                                break;
                            } while(false);
                            float _1489 = _1173 * 0.3535533845424652099609375;
                            float _1539;
                            do
                            {
                                vec2 _1497 = clamp(_477 + (vec2(_1489 * _1178, _1489 * _1177) * _1437), vec2(0.0), vec2(1.0));
                                if (_1106)
                                {
                                    _1539 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1497, _374));
                                    break;
                                }
                                if (_475 == 1u)
                                {
                                    _1539 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1497, _374));
                                    break;
                                }
                                if (_475 == 2u)
                                {
                                    _1539 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1497, _374));
                                    break;
                                }
                                if (_475 == 3u)
                                {
                                    _1539 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1497, _374));
                                    break;
                                }
                                if (_475 == 4u)
                                {
                                    _1539 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1497, _374));
                                    break;
                                }
                                if (_475 == 5u)
                                {
                                    _1539 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1497, _374));
                                    break;
                                }
                                _1539 = 0.0;
                                break;
                            } while(false);
                            float _1541 = _1257 * 0.3535533845424652099609375;
                            float _1591;
                            do
                            {
                                vec2 _1549 = clamp(_477 + (vec2(_1541 * _1262, _1541 * _1261) * _1437), vec2(0.0), vec2(1.0));
                                if (_1106)
                                {
                                    _1591 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1549, _374));
                                    break;
                                }
                                if (_475 == 1u)
                                {
                                    _1591 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1549, _374));
                                    break;
                                }
                                if (_475 == 2u)
                                {
                                    _1591 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1549, _374));
                                    break;
                                }
                                if (_475 == 3u)
                                {
                                    _1591 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1549, _374));
                                    break;
                                }
                                if (_475 == 4u)
                                {
                                    _1591 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1549, _374));
                                    break;
                                }
                                if (_475 == 5u)
                                {
                                    _1591 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1549, _374));
                                    break;
                                }
                                _1591 = 0.0;
                                break;
                            } while(false);
                            float _1593 = _1341 * 0.3535533845424652099609375;
                            float _1643;
                            do
                            {
                                vec2 _1601 = clamp(_477 + (vec2(_1593 * _1346, _1593 * _1345) * _1437), vec2(0.0), vec2(1.0));
                                if (_1106)
                                {
                                    _1643 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1601, _374));
                                    break;
                                }
                                if (_475 == 1u)
                                {
                                    _1643 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1601, _374));
                                    break;
                                }
                                if (_475 == 2u)
                                {
                                    _1643 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1601, _374));
                                    break;
                                }
                                if (_475 == 3u)
                                {
                                    _1643 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1601, _374));
                                    break;
                                }
                                if (_475 == 4u)
                                {
                                    _1643 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1601, _374));
                                    break;
                                }
                                if (_475 == 5u)
                                {
                                    _1643 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1601, _374));
                                    break;
                                }
                                _1643 = 0.0;
                                break;
                            } while(false);
                            float _1646 = float(uint(4));
                            float _1649 = sqrt(_1646 + 0.5) * 0.3535533845424652099609375;
                            float _1651 = (_1646 * 2.400000095367431640625) + _1087;
                            float _1703;
                            do
                            {
                                vec2 _1661 = clamp(_477 + (vec2(_1649 * cos(_1651), _1649 * sin(_1651)) * _1437), vec2(0.0), vec2(1.0));
                                if (_1106)
                                {
                                    _1703 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1661, _374));
                                    break;
                                }
                                if (_475 == 1u)
                                {
                                    _1703 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1661, _374));
                                    break;
                                }
                                if (_475 == 2u)
                                {
                                    _1703 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1661, _374));
                                    break;
                                }
                                if (_475 == 3u)
                                {
                                    _1703 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1661, _374));
                                    break;
                                }
                                if (_475 == 4u)
                                {
                                    _1703 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1661, _374));
                                    break;
                                }
                                if (_475 == 5u)
                                {
                                    _1703 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1661, _374));
                                    break;
                                }
                                _1703 = 0.0;
                                break;
                            } while(false);
                            float _1706 = float(uint(5));
                            float _1709 = sqrt(_1706 + 0.5) * 0.3535533845424652099609375;
                            float _1711 = (_1706 * 2.400000095367431640625) + _1087;
                            float _1763;
                            do
                            {
                                vec2 _1721 = clamp(_477 + (vec2(_1709 * cos(_1711), _1709 * sin(_1711)) * _1437), vec2(0.0), vec2(1.0));
                                if (_1106)
                                {
                                    _1763 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1721, _374));
                                    break;
                                }
                                if (_475 == 1u)
                                {
                                    _1763 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1721, _374));
                                    break;
                                }
                                if (_475 == 2u)
                                {
                                    _1763 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1721, _374));
                                    break;
                                }
                                if (_475 == 3u)
                                {
                                    _1763 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1721, _374));
                                    break;
                                }
                                if (_475 == 4u)
                                {
                                    _1763 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1721, _374));
                                    break;
                                }
                                if (_475 == 5u)
                                {
                                    _1763 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1721, _374));
                                    break;
                                }
                                _1763 = 0.0;
                                break;
                            } while(false);
                            float _1766 = float(uint(6));
                            float _1769 = sqrt(_1766 + 0.5) * 0.3535533845424652099609375;
                            float _1771 = (_1766 * 2.400000095367431640625) + _1087;
                            float _1823;
                            do
                            {
                                vec2 _1781 = clamp(_477 + (vec2(_1769 * cos(_1771), _1769 * sin(_1771)) * _1437), vec2(0.0), vec2(1.0));
                                if (_1106)
                                {
                                    _1823 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1781, _374));
                                    break;
                                }
                                if (_475 == 1u)
                                {
                                    _1823 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1781, _374));
                                    break;
                                }
                                if (_475 == 2u)
                                {
                                    _1823 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1781, _374));
                                    break;
                                }
                                if (_475 == 3u)
                                {
                                    _1823 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1781, _374));
                                    break;
                                }
                                if (_475 == 4u)
                                {
                                    _1823 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1781, _374));
                                    break;
                                }
                                if (_475 == 5u)
                                {
                                    _1823 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1781, _374));
                                    break;
                                }
                                _1823 = 0.0;
                                break;
                            } while(false);
                            float _1826 = float(uint(7));
                            float _1829 = sqrt(_1826 + 0.5) * 0.3535533845424652099609375;
                            float _1831 = (_1826 * 2.400000095367431640625) + _1087;
                            float _1883;
                            do
                            {
                                vec2 _1841 = clamp(_477 + (vec2(_1829 * cos(_1831), _1829 * sin(_1831)) * _1437), vec2(0.0), vec2(1.0));
                                if (_1106)
                                {
                                    _1883 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1841, _374));
                                    break;
                                }
                                if (_475 == 1u)
                                {
                                    _1883 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1841, _374));
                                    break;
                                }
                                if (_475 == 2u)
                                {
                                    _1883 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1841, _374));
                                    break;
                                }
                                if (_475 == 3u)
                                {
                                    _1883 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1841, _374));
                                    break;
                                }
                                if (_475 == 4u)
                                {
                                    _1883 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1841, _374));
                                    break;
                                }
                                if (_475 == 5u)
                                {
                                    _1883 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1841, _374));
                                    break;
                                }
                                _1883 = 0.0;
                                break;
                            } while(false);
                            _1886 = (((((((_1488 + _1539) + _1591) + _1643) + _1703) + _1763) + _1823) + _1883) * 0.125;
                        }
                        else
                        {
                            _1886 = 0.0;
                        }
                        _1887 = _1886;
                    }
                    else
                    {
                        bool _500;
                        float _542;
                        do
                        {
                            vec2 _499 = clamp(_477 + (vec2(-0.326211988925933837890625, -0.40580499172210693359375) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize), vec2(0.0), vec2(1.0));
                            _500 = _475 == 0u;
                            if (_500)
                            {
                                _542 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_499, _374));
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _542 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_499, _374));
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _542 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_499, _374));
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _542 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_499, _374));
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _542 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_499, _374));
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _542 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_499, _374));
                                break;
                            }
                            _542 = 0.0;
                            break;
                        } while(false);
                        float _590;
                        do
                        {
                            vec2 _548 = clamp(_477 + (vec2(-0.840143978595733642578125, -0.0735799968242645263671875) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize), vec2(0.0), vec2(1.0));
                            if (_500)
                            {
                                _590 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_548, _374));
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _590 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_548, _374));
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _590 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_548, _374));
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _590 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_548, _374));
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _590 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_548, _374));
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _590 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_548, _374));
                                break;
                            }
                            _590 = 0.0;
                            break;
                        } while(false);
                        float _639;
                        do
                        {
                            vec2 _597 = clamp(_477 + (vec2(-0.695913970470428466796875, 0.45713698863983154296875) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize), vec2(0.0), vec2(1.0));
                            if (_500)
                            {
                                _639 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_597, _374));
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _639 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_597, _374));
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _639 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_597, _374));
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _639 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_597, _374));
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _639 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_597, _374));
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _639 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_597, _374));
                                break;
                            }
                            _639 = 0.0;
                            break;
                        } while(false);
                        float _688;
                        do
                        {
                            vec2 _646 = clamp(_477 + (vec2(-0.203345000743865966796875, 0.62071597576141357421875) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize), vec2(0.0), vec2(1.0));
                            if (_500)
                            {
                                _688 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_646, _374));
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _688 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_646, _374));
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _688 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_646, _374));
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _688 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_646, _374));
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _688 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_646, _374));
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _688 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_646, _374));
                                break;
                            }
                            _688 = 0.0;
                            break;
                        } while(false);
                        float _737;
                        do
                        {
                            vec2 _695 = clamp(_477 + (vec2(0.96233999729156494140625, -0.194983005523681640625) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize), vec2(0.0), vec2(1.0));
                            if (_500)
                            {
                                _737 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_695, _374));
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _737 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_695, _374));
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _737 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_695, _374));
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _737 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_695, _374));
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _737 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_695, _374));
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _737 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_695, _374));
                                break;
                            }
                            _737 = 0.0;
                            break;
                        } while(false);
                        float _786;
                        do
                        {
                            vec2 _744 = clamp(_477 + (vec2(0.4734340012073516845703125, -0.4800260066986083984375) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize), vec2(0.0), vec2(1.0));
                            if (_500)
                            {
                                _786 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_744, _374));
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _786 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_744, _374));
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _786 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_744, _374));
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _786 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_744, _374));
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _786 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_744, _374));
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _786 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_744, _374));
                                break;
                            }
                            _786 = 0.0;
                            break;
                        } while(false);
                        float _835;
                        do
                        {
                            vec2 _793 = clamp(_477 + (vec2(0.51945602893829345703125, 0.76702201366424560546875) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize), vec2(0.0), vec2(1.0));
                            if (_500)
                            {
                                _835 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_793, _374));
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _835 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_793, _374));
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _835 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_793, _374));
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _835 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_793, _374));
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _835 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_793, _374));
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _835 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_793, _374));
                                break;
                            }
                            _835 = 0.0;
                            break;
                        } while(false);
                        float _884;
                        do
                        {
                            vec2 _842 = clamp(_477 + (vec2(0.18546099960803985595703125, -0.89312398433685302734375) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize), vec2(0.0), vec2(1.0));
                            if (_500)
                            {
                                _884 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_842, _374));
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _884 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_842, _374));
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _884 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_842, _374));
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _884 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_842, _374));
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _884 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_842, _374));
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _884 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_842, _374));
                                break;
                            }
                            _884 = 0.0;
                            break;
                        } while(false);
                        float _933;
                        do
                        {
                            vec2 _891 = clamp(_477 + (vec2(0.507430970668792724609375, 0.064424999058246612548828125) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize), vec2(0.0), vec2(1.0));
                            if (_500)
                            {
                                _933 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_891, _374));
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _933 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_891, _374));
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _933 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_891, _374));
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _933 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_891, _374));
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _933 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_891, _374));
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _933 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_891, _374));
                                break;
                            }
                            _933 = 0.0;
                            break;
                        } while(false);
                        float _982;
                        do
                        {
                            vec2 _940 = clamp(_477 + (vec2(0.896420001983642578125, 0.412458002567291259765625) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize), vec2(0.0), vec2(1.0));
                            if (_500)
                            {
                                _982 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_940, _374));
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _982 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_940, _374));
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _982 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_940, _374));
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _982 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_940, _374));
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _982 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_940, _374));
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _982 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_940, _374));
                                break;
                            }
                            _982 = 0.0;
                            break;
                        } while(false);
                        float _1031;
                        do
                        {
                            vec2 _989 = clamp(_477 + (vec2(-0.321940004825592041015625, -0.932614982128143310546875) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize), vec2(0.0), vec2(1.0));
                            if (_500)
                            {
                                _1031 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_989, _374));
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _1031 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_989, _374));
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _1031 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_989, _374));
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _1031 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_989, _374));
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _1031 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_989, _374));
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _1031 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_989, _374));
                                break;
                            }
                            _1031 = 0.0;
                            break;
                        } while(false);
                        float _1080;
                        do
                        {
                            vec2 _1038 = clamp(_477 + (vec2(-0.7915589809417724609375, -0.59770500659942626953125) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_axFace[_475].fInvSize), vec2(0.0), vec2(1.0));
                            if (_500)
                            {
                                _1080 = texture(sampler2DShadow(g_xShadowCubeMapPosX, g_xShadowCmpSampler), vec3(_1038, _374));
                                break;
                            }
                            if (_475 == 1u)
                            {
                                _1080 = texture(sampler2DShadow(g_xShadowCubeMapNegX, g_xShadowCmpSampler), vec3(_1038, _374));
                                break;
                            }
                            if (_475 == 2u)
                            {
                                _1080 = texture(sampler2DShadow(g_xShadowCubeMapPosY, g_xShadowCmpSampler), vec3(_1038, _374));
                                break;
                            }
                            if (_475 == 3u)
                            {
                                _1080 = texture(sampler2DShadow(g_xShadowCubeMapNegY, g_xShadowCmpSampler), vec3(_1038, _374));
                                break;
                            }
                            if (_475 == 4u)
                            {
                                _1080 = texture(sampler2DShadow(g_xShadowCubeMapPosZ, g_xShadowCmpSampler), vec3(_1038, _374));
                                break;
                            }
                            if (_475 == 5u)
                            {
                                _1080 = texture(sampler2DShadow(g_xShadowCubeMapNegZ, g_xShadowCmpSampler), vec3(_1038, _374));
                                break;
                            }
                            _1080 = 0.0;
                            break;
                        } while(false);
                        _1887 = (((((((((((_542 + _590) + _639) + _688) + _737) + _786) + _835) + _884) + _933) + _982) + _1031) + _1080) * 0.083333335816860198974609375;
                    }
                    _1888 = _1887;
                    break;
                }
            } while(false);
            vec4 _1891 = vec4(0.0, 0.0, 0.0, 1.0);
            _1891.w = 1.0 - (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fShadowStrength * _1888);
            _1892 = _1891;
        }
        else
        {
            _1892 = vec4(0.0, 0.0, 0.0, 1.0);
        }
        if (_1892.w > 0.0)
        {
            vec3 _1897 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightColour * _349;
            vec3 _1924;
            if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_bUseProjCube != 0u)
            {
                vec3 _1907 = mat3(g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightOrientation[0].xyz, g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightOrientation[1].xyz, g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_xLightOrientation[2].xyz) * _333;
                vec4 _1910 = texture(samplerCube(g_xProjCubeMap, g_xTrilinearClamp), _1907);
                vec3 _1911 = _1910.xyz;
                vec3 _1922;
                if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_bUseDualProjCube != 0u)
                {
                    _1922 = mix(_1911, texture(samplerCube(g_xProjCubeMapBlurred, g_xTrilinearClamp), _1907).xyz, vec3(min((_331 / g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRange) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fRcpProjCubeLerpDist, 1.0)));
                }
                else
                {
                    _1922 = _1911;
                }
                _1924 = _1897 * _1922;
            }
            else
            {
                _1924 = _1897;
            }
            vec3 _1926 = normalize(FrameConstants.g_xCamPos - _248);
            float _1944;
            vec3 _1945;
            if (g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRadius > 0.0)
            {
                vec3 _1931 = -reflect(_1926, _258);
                vec3 _1934 = (_1931 * dot(_330, _1931)) - _330;
                _1944 = g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRadius * inversesqrt(dot(_330, _330));
                _1945 = normalize(_330 + (_1934 * clamp(g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fLightRadius / length(_1934), 0.0, 1.0)));
            }
            else
            {
                _1944 = 0.0;
                _1945 = _333;
            }
            float _1947 = (_261 != 0.01960784383118152618408203125) ? _262 : 0.0;
            vec3 _1949 = normalize(_1926 + _1945);
            float _1951 = clamp(dot(_258, _1926), 0.0, 1.0);
            float _1953 = clamp(dot(_258, _1949), 0.0, 1.0);
            float _1955 = clamp(dot(_1926, _1949), 0.0, 1.0);
            bool _1956 = _261 == 0.01568627543747425079345703125;
            vec3 _1993;
            vec3 _1994;
            vec3 _1995;
            if (_1956)
            {
                _1993 = vec3(0.046999998390674591064453125);
                _1994 = vec3(_354);
                _1995 = _259;
            }
            else
            {
                vec3 _1989;
                vec3 _1990;
                vec3 _1991;
                if (_261 == 0.007843137718737125396728515625)
                {
                    _1989 = vec3(0.0280000008642673492431640625);
                    _1990 = vec3(_354);
                    _1991 = _259;
                }
                else
                {
                    vec3 _1985;
                    vec3 _1986;
                    vec3 _1987;
                    if (_261 == 0.0039215688593685626983642578125)
                    {
                        _1985 = vec3(0.0280000008642673492431640625);
                        _1986 = vec3((clamp((clamp(dot(_333, -_258), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_333, -_1926), 0.0, 1.0), 7.0), 0.0, 1.0) * _1947) + _354);
                        _1987 = _259;
                    }
                    else
                    {
                        vec3 _1968 = vec3(_1947);
                        _1985 = mix(vec3(0.039999999105930328369140625), _259, _1968);
                        _1986 = vec3(_354);
                        _1987 = mix(_259, vec3(0.0), _1968);
                    }
                    _1989 = _1985;
                    _1990 = _1986;
                    _1991 = _1987;
                }
                _1993 = _1989;
                _1994 = _1990;
                _1995 = _1991;
            }
            vec3 _1996 = _1994 * _1995;
            float _1997 = max(_260, 0.0500000007450580596923828125);
            float _1998 = _1997 * _1997;
            vec3 _2111;
            if (_1956)
            {
                float _2049 = clamp(1.0 - _260, 0.0, 1.0);
                float _2051 = ((_262 * 2.0) - 1.0) * 0.300000011920928955078125;
                float _2057 = pow(_2049, 8.0) * 500.0;
                float _2060 = pow(_2049, 6.0) * 2000.0;
                float _2074 = dot(normalize(_276 + (_258 * ((-0.0500000007450580596923828125) + _2051))), _1949);
                float _2094 = dot(normalize(_276 + (_258 * ((0.0500000007450580596923828125 + _2051) - (_2049 * 0.0500000007450580596923828125)))), _1949);
                _2111 = _1996 + ((((vec3(((smoothstep(-1.0, 0.0, _2074) * pow(sqrt(clamp(1.0 - (_2074 * _2074), 0.0, 1.0)), 20.0 + _2057)) * ((((_262 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _2049))) * ((_2057 + 28.0) * 0.03978873789310455322265625)) * (_259 / vec3(max(dot(_259, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _2094) * pow(sqrt(clamp(1.0 - (_2094 * _2094), 0.0, 1.0)), 50.0 + _2060)) * ((((_262 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _2049)) * ((_2060 + 58.0) * 0.03978873789310455322265625))) * _354) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fSpecularStrength);
            }
            else
            {
                float _2004 = clamp(_1944 * (1.0 - _1998), 0.0, 1.0);
                float _2024;
                if (_2004 > 0.0)
                {
                    float _2015 = _1998 * _1998;
                    _2024 = _2015 / (_2015 + (((_2004 * 0.25) * ((3.0 * _1998) + _2004)) / (_1955 + 0.001000000047497451305389404296875)));
                }
                else
                {
                    _2024 = 1.0;
                }
                float _2025 = 2.0 - _1998;
                float _2032 = _1998 * _1998;
                float _2036 = (((_1953 * _2032) - _1953) * _1953) + 1.0;
                _2111 = _1996 + (((((vec3(1.0) - _1993) * exp2((((-5.554729938507080078125) * _1955) - 6.9831600189208984375) * _1955)) + _1993) * ((((1.0 / (((_1951 * _2025) + _1998) * ((_354 * _2025) + _1998))) * (_2032 * (1.0 / (_2036 * _2036)))) * _354) * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fSpecularStrength)) * _2024);
            }
            vec3 _2161;
            if (_261 == 0.01960784383118152618408203125)
            {
                bool _2115 = _262 > 0.4980392158031463623046875;
                vec3 _2133 = sqrt(abs(_1995));
                float _2147 = 0.5 / _1998;
                _2161 = mix(_2111, (_1996 + (((((vec3(1.0) - _2133) * exp2((((-5.554729938507080078125) * _1955) - 6.9831600189208984375) * _1955)) + _2133) * ((1.0 / ((4.0 * ((_354 + _1951) - (_354 * _1951))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _2147) * pow(1.0 - (_1953 * _1953), _2147)))) * (_354 * g_axPhotons._m0[PhotonIndex.g_uPhotonIndex].m_fSpecularStrength))) + (_1995 * (clamp((clamp(dot(_333, -_258), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(_333, -_1926), 0.0, 1.0), 7.0), 0.0, 1.0) * (_2115 ? 0.5 : 0.0))), vec3((_2115 ? (_262 - 0.501960813999176025390625) : _262) * 2.007874011993408203125));
            }
            else
            {
                _2161 = _2111;
            }
            vec3 _2164 = min(_2161 * _1924, vec3(1000.0)).xyz;
            bool _2165 = _261 == 0.007843137718737125396728515625;
            vec3 _2176;
            if (_2165)
            {
                _2176 = _2164 + (((_259 * vec3(0.0194447897374629974365234375, 1.6805228369776159524917602539063e-05, 3.1907067266079280898338765837252e-11)) * clamp(0.300000011920928955078125 + dot(_333, -_258), 0.0, 1.0)) * _1924);
            }
            else
            {
                _2176 = _2164;
            }
            vec3 _2189;
            if (_2165 && (_229 < 3.0))
            {
                _2189 = _2176.xyz * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(_229 - 2.0, 0.0)));
            }
            else
            {
                _2189 = _2176.xyz;
            }
            _2191 = vec4(_2189.x, _2189.y, _2189.z, _1892.w);
            _2192 = true;
            break;
        }
        _2191 = _1892;
        _2192 = false;
        break;
    } while(false);
    if (false == _2192)
    {
        discard;
    }
    out_var_SV_Target = _2191;
}


