#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require

struct CSMSingleMapConstantsStruct
{
    mat4 g_xShadowViewProj;
    mat4 g_xInvShadowByViewProj;
    mat4 g_xScaledInvShadowByViewProj;
    float g_fSize;
    float g_fAreaScale;
    float g_fBlendArea;
    float g_fStrength;
    uint g_uMap;
    float g_fBiasFactor;
    float g_fNearClip;
    float g_fFarClip;
};


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 2, binding = 6, scalar) uniform type_FogParams
{
    vec3 g_xFogColour;
    float g_fFogNearPlane;
    float g_fRcpFogVolumeDepth;
    float g_fMaxFogDensity;
    uint g_bFogEnabled;
    uint g_bTexturedFogEnabled;
    float g_fHorizonOffset;
    float g_fHorizonScale;
    float g_fTextureHDRScale;
    float g_fTextureUOffset;
    float g_fDensityPower;
    float g_fVerticalTextureScale;
    float g_fHeightFogStrength;
    float g_fHeightFogFalloff;
    float g_fHeightFogHeight;
    vec3 g_xHeightFogColour;
    uint g_bHeightFogEnabled;
    float g_fHeightFogMaxOpacity;
    float g_fHeightFogStartDistance;
    float g_fCosScatteringAngle;
    vec3 g_xSunDirection;
    float g_fDirectScatterStartDistance;
    float g_fDirectScatterFadeDistance;
    vec3 g_xDirectColour;
} FogParams;

layout(set = 2, binding = 7, std140) uniform type_ProjectedCloudParams
{
    uint g_bSunlightCloudsEnabled;
    float g_fSunlightCloudsVelocityX;
    float g_fSunlightCloudsVelocityY;
    float g_fSunlightCloudsScale;
    vec4 g_xSunlightCloudsProjectionDir;
} ProjectedCloudParams;

layout(set = 1, binding = 9, scalar) uniform type_PFXConstants
{
    float g_fMotionVectorStrength;
    vec2 g_xRcpSSTextureDims;
    float g_fPad1;
    uint g_bCrossfadeTextureFrames;
    uint g_bVolumetric;
    uint g_bUseFogging;
    uint g_bUseCSMLighting;
    uint g_bProjectorShadow;
    uint g_bRGBRemap;
    uint g_bAlphaRemap;
    uint g_bMotionVector;
    uint g_bUsePerPixelHeightFog;
} PFXConstants;

layout(set = 1, binding = 10, std140) uniform type_PFXLighting
{
    vec3 g_xSunColour;
    float g_fPad;
    vec3 g_xShadowColour;
    float g_fPad2;
} PFXLighting;

layout(set = 2, binding = 11, std140) uniform type_CSMLevelSizedMapParams
{
    layout(row_major) CSMSingleMapConstantsStruct g_xCSMLevelSizedParams;
} CSMLevelSizedMapParams;

layout(set = 0, binding = 2) uniform sampler g_xBilinearWrap;
layout(set = 0, binding = 5) uniform sampler g_xTrilinearClamp;
layout(set = 0, binding = 6) uniform sampler g_xFilteredWrap;
layout(set = 0, binding = 7) uniform sampler g_xFilteredClamp;
layout(set = 0, binding = 8) uniform samplerShadow g_xShadowCmpSampler;
layout(set = 2, binding = 49) uniform texture1D g_xFogTexture;
layout(set = 2, binding = 100) uniform TEXTURE_2D g_xLighting_Clouds;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xEffectTexture;
layout(set = 2, binding = 50) uniform TEXTURE_2D g_xDepthStencil;
layout(set = 2, binding = 54) uniform texture2D g_xShadowTexture;
layout(set = 1, binding = 58) uniform TEXTURE_2D g_xRGBRemapTexture;
layout(set = 1, binding = 59) uniform TEXTURE_2D g_xAlphaRemapTexture;
layout(set = 1, binding = 60) uniform TEXTURE_2D g_xMotionVectorTexture;

layout(location = 0) in vec4 in_var_COLOR0;
layout(location = 1) in vec4 in_var_TEXCOORD0;
layout(location = 2) in vec4 in_var_TEXCOORD1;
layout(location = 3) in vec4 in_var_TEXCOORD2;
layout(location = 0) out vec4 out_var_SV_Target;

float _117;

void main()
{
    vec4 _703;
    do
    {
        float _127 = 1.0 / gl_FragCoord.w;
        bool _133 = PFXConstants.g_bMotionVector != 0u;
        vec2 _161;
        vec2 _162;
        if (_133)
        {
            vec4 _148 = (vec4(SAMPLE_TEXTURE(SAMPLER_2D(g_xMotionVectorTexture, g_xFilteredWrap), in_var_TEXCOORD0.xy).xy, SAMPLE_TEXTURE(SAMPLER_2D(g_xMotionVectorTexture, g_xFilteredWrap), in_var_TEXCOORD0.zw).xy) * 2.0) - vec4(1.0);
            _161 = in_var_TEXCOORD0.zw - ((_148.zw * PFXConstants.g_fMotionVectorStrength) * in_var_TEXCOORD1.w);
            _162 = in_var_TEXCOORD0.xy + ((_148.xy * PFXConstants.g_fMotionVectorStrength) * (1.0 - in_var_TEXCOORD1.w));
        }
        else
        {
            _161 = in_var_TEXCOORD0.zw;
            _162 = in_var_TEXCOORD0.xy;
        }
        vec4 _166 = SAMPLE_TEXTURE(SAMPLER_2D(g_xEffectTexture, g_xFilteredWrap), _162);
        vec4 _168 = SAMPLE_TEXTURE(SAMPLER_2D(g_xEffectTexture, g_xFilteredWrap), _161);
        vec4 _179;
        if ((PFXConstants.g_bCrossfadeTextureFrames != 0u) || _133)
        {
            _179 = mix(_168, _166, vec4(in_var_TEXCOORD1.w));
        }
        else
        {
            _179 = _166;
        }
        vec4 _194;
        if (PFXConstants.g_bAlphaRemap != 0u)
        {
            vec4 _193 = _179;
            _193.w = TEXTURE_LOD(SAMPLER_2D(g_xAlphaRemapTexture, g_xFilteredClamp), vec2(_179.w, in_var_TEXCOORD1.z), 0.0).x;
            _194 = _193;
        }
        else
        {
            _194 = _179;
        }
        if (_194.w < 0.001000000047497451305389404296875)
        {
            discard;
        }
        vec4 _219;
        if (PFXConstants.g_bRGBRemap != 0u)
        {
            vec4 _217 = TEXTURE_LOD(SAMPLER_2D(g_xRGBRemapTexture, g_xFilteredClamp), vec2(mix((1.05499994754791259765625 * pow(abs(_194.x), 0.4166666567325592041015625)) - 0.054999999701976776123046875, _194.x * 12.9200000762939453125, step(_194.x, 0.003130800090730190277099609375)), in_var_TEXCOORD1.y), 0.0);
            _219 = vec4(_217.x, _217.y, _217.z, _194.w);
        }
        else
        {
            _219 = _194;
        }
        vec4 _220 = in_var_COLOR0 * _219;
        vec4 _330;
        if (PFXConstants.g_bUseCSMLighting != 0u)
        {
            vec4 _246 = vec4((FrameConstants.g_xCameraTanHalfFOV * _127) * (((vec3(gl_FragCoord.xy, _127).xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _127, 1.0) * FrameConstants.g_xInvView;
            vec4 _249 = _246 * CSMLevelSizedMapParams.g_xCSMLevelSizedParams.g_xShadowViewProj;
            vec2 _252 = (_249.xy * 0.5) + vec2(0.5);
            vec3 _256 = vec3(_252.x, _252.y, _249.z);
            _256.y = 1.0 - _252.y;
            float _262 = texture(sampler2DShadow(g_xShadowTexture, g_xShadowCmpSampler), vec3(_256.xy, _249.z));
            float _318;
            if (ProjectedCloudParams.g_bSunlightCloudsEnabled != 0u)
            {
                float _317;
                if (abs(ProjectedCloudParams.g_xSunlightCloudsProjectionDir.y) > 9.9999999747524270787835121154785e-07)
                {
                    vec3 _280 = _246.xyz;
                    float _283 = (vec3(0.0, -10000.0, 0.0) - _280).y / ProjectedCloudParams.g_xSunlightCloudsProjectionDir.y;
                    float _316;
                    if (_283 > 0.0)
                    {
                        vec3 _288 = _280 + (ProjectedCloudParams.g_xSunlightCloudsProjectionDir.xyz * _283);
                        vec2 _297 = _288.xz;
                        _297.x = _288.x + (FrameConstants.g_fScaledTime * ProjectedCloudParams.g_fSunlightCloudsVelocityX);
                        vec2 _303 = _297;
                        _303.y = _288.z + (FrameConstants.g_fScaledTime * ProjectedCloudParams.g_fSunlightCloudsVelocityY);
                        vec2 _307 = _303 / vec2(ProjectedCloudParams.g_fSunlightCloudsScale);
                        vec2 _310 = _307;
                        _310.x = _307.x * (-1.0);
                        _316 = TEXTURE_LOD(SAMPLER_2D(g_xLighting_Clouds, g_xBilinearWrap), _310, 0.0).x;
                    }
                    else
                    {
                        _316 = 1.0;
                    }
                    _317 = _316;
                }
                else
                {
                    _317 = 1.0;
                }
                _318 = _317;
            }
            else
            {
                _318 = 1.0;
            }
            vec3 _328 = _220.xyz * mix(PFXLighting.g_xShadowColour, PFXLighting.g_xSunColour, vec3(clamp(min(1.0 - (_262 * CSMLevelSizedMapParams.g_xCSMLevelSizedParams.g_fStrength), _318), 0.0, 1.0)));
            _330 = vec4(_328.x, _328.y, _328.z, _220.w);
        }
        else
        {
            _330 = _220;
        }
        vec4 _362;
        if (PFXConstants.g_bVolumetric != 0u)
        {
            float _357 = _330.w * clamp(abs((FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xDepthStencil, ivec3(int(gl_FragCoord.x), int(gl_FragCoord.y), 0).xy, 0).x)) - _127) / in_var_TEXCOORD1.x, 0.0, 1.0);
            vec4 _358 = _330;
            _358.w = _357;
            if (_357 < 0.001000000047497451305389404296875)
            {
                discard;
            }
            _362 = _358;
        }
        else
        {
            _362 = _330;
        }
        if (PFXConstants.g_bUseFogging != 0u)
        {
            if (PFXConstants.g_bUsePerPixelHeightFog != 0u)
            {
                vec4 _694;
                if (FogParams.g_bHeightFogEnabled != 0u)
                {
                    vec3 _556 = (vec4((FrameConstants.g_xCameraTanHalfFOV * _127) * (((gl_FragCoord.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _127, 1.0) * FrameConstants.g_xInvView).xyz - FrameConstants.g_xCamPos;
                    float _557 = length(_556);
                    vec3 _559 = -normalize(_556);
                    vec4 _663;
                    if (_557 > FogParams.g_fHeightFogStartDistance)
                    {
                        float _567 = max(_557 - FogParams.g_fHeightFogStartDistance, 0.0);
                        float _591;
                        if (_557 >= FrameConstants.g_xCameraFarClipValues.x)
                        {
                            _591 = float(int(sign(_559.y))) * 1.57079601287841796875;
                        }
                        else
                        {
                            _591 = atan((((_559.y * _567) - FrameConstants.g_xCamPos.y) - FogParams.g_fHeightFogHeight) / FogParams.g_fHeightFogFalloff);
                        }
                        float _592 = _559.y;
                        float _627;
                        if (abs(_592) <= 9.9999999747524270787835121154785e-07)
                        {
                            float _622 = (-FrameConstants.g_xCamPos.y) - FogParams.g_fHeightFogHeight;
                            _627 = (((FogParams.g_fHeightFogStrength * FogParams.g_fHeightFogFalloff) * _567) * FogParams.g_fHeightFogFalloff) / ((_622 * _622) + (FogParams.g_fHeightFogFalloff * FogParams.g_fHeightFogFalloff));
                        }
                        else
                        {
                            _627 = ((FogParams.g_fHeightFogStrength * FogParams.g_fHeightFogFalloff) * (_591 - atan(((-FogParams.g_fHeightFogHeight) - FrameConstants.g_xCamPos.y) / FogParams.g_fHeightFogFalloff))) / _592;
                        }
                        float _638 = clamp(max(-dot(FogParams.g_xSunDirection, _559), 0.0) - FogParams.g_fCosScatteringAngle, 0.0, 1.0) / (1.0 - FogParams.g_fCosScatteringAngle);
                        _663 = vec4(FogParams.g_xHeightFogColour + ((FogParams.g_xDirectColour * clamp(max(_557 - FogParams.g_fDirectScatterStartDistance, 0.0) / FogParams.g_fDirectScatterFadeDistance, 0.0, 1.0)) * (_638 * _638)), clamp(_627, 0.0, 1.0) * FogParams.g_fHeightFogMaxOpacity);
                    }
                    else
                    {
                        _663 = vec4(0.0);
                    }
                    vec4 _693;
                    if (FogParams.g_bTexturedFogEnabled != 0u)
                    {
                        float _674 = (_127 - FogParams.g_fFogNearPlane) * FogParams.g_fRcpFogVolumeDepth;
                        _693 = vec4(_117, _117, _117, clamp(_663.w + ((FogParams.g_fDensityPower > 0.0) ? (pow(clamp(_674, 0.0, 1.0), FogParams.g_fDensityPower) * FogParams.g_fMaxFogDensity) : texture(sampler1D(g_xFogTexture, g_xTrilinearClamp), _674).x), 0.0, 1.0));
                    }
                    else
                    {
                        _693 = _663;
                    }
                    _694 = _693;
                }
                else
                {
                    vec4 _545;
                    if (FogParams.g_bFogEnabled != 0u)
                    {
                        float _529 = (_127 - FogParams.g_fFogNearPlane) * FogParams.g_fRcpFogVolumeDepth;
                        _545 = vec4(_117, _117, _117, (FogParams.g_fDensityPower > 0.0) ? (pow(clamp(_529, 0.0, 1.0), FogParams.g_fDensityPower) * FogParams.g_fMaxFogDensity) : texture(sampler1D(g_xFogTexture, g_xTrilinearClamp), _529).x);
                    }
                    else
                    {
                        vec4 _523;
                        if (FogParams.g_bTexturedFogEnabled != 0u)
                        {
                            float _507 = (_127 - FogParams.g_fFogNearPlane) * FogParams.g_fRcpFogVolumeDepth;
                            _523 = vec4(_117, _117, _117, (FogParams.g_fDensityPower > 0.0) ? (pow(clamp(_507, 0.0, 1.0), FogParams.g_fDensityPower) * FogParams.g_fMaxFogDensity) : texture(sampler1D(g_xFogTexture, g_xTrilinearClamp), _507).x);
                        }
                        else
                        {
                            _523 = vec4(0.0);
                        }
                        _545 = _523;
                    }
                    _694 = _545;
                }
                _703 = vec4(_362.xyz, _362.w * (1.0 - _694.w));
                break;
            }
            else
            {
                vec4 _476;
                if (FogParams.g_bHeightFogEnabled != 0u)
                {
                    vec4 _475;
                    if (FogParams.g_bTexturedFogEnabled != 0u)
                    {
                        float _456 = (_127 - FogParams.g_fFogNearPlane) * FogParams.g_fRcpFogVolumeDepth;
                        _475 = vec4(_117, _117, _117, clamp(in_var_TEXCOORD2.w + ((FogParams.g_fDensityPower > 0.0) ? (pow(clamp(_456, 0.0, 1.0), FogParams.g_fDensityPower) * FogParams.g_fMaxFogDensity) : texture(sampler1D(g_xFogTexture, g_xTrilinearClamp), _456).x), 0.0, 1.0));
                    }
                    else
                    {
                        _475 = in_var_TEXCOORD2;
                    }
                    _476 = _475;
                }
                else
                {
                    vec4 _445;
                    if (FogParams.g_bFogEnabled != 0u)
                    {
                        float _429 = (_127 - FogParams.g_fFogNearPlane) * FogParams.g_fRcpFogVolumeDepth;
                        _445 = vec4(_117, _117, _117, (FogParams.g_fDensityPower > 0.0) ? (pow(clamp(_429, 0.0, 1.0), FogParams.g_fDensityPower) * FogParams.g_fMaxFogDensity) : texture(sampler1D(g_xFogTexture, g_xTrilinearClamp), _429).x);
                    }
                    else
                    {
                        vec4 _423;
                        if (FogParams.g_bTexturedFogEnabled != 0u)
                        {
                            float _407 = (_127 - FogParams.g_fFogNearPlane) * FogParams.g_fRcpFogVolumeDepth;
                            _423 = vec4(_117, _117, _117, (FogParams.g_fDensityPower > 0.0) ? (pow(clamp(_407, 0.0, 1.0), FogParams.g_fDensityPower) * FogParams.g_fMaxFogDensity) : texture(sampler1D(g_xFogTexture, g_xTrilinearClamp), _407).x);
                        }
                        else
                        {
                            _423 = vec4(0.0);
                        }
                        _445 = _423;
                    }
                    _476 = _445;
                }
                _703 = vec4(_362.xyz, _362.w * (1.0 - _476.w));
                break;
            }
        }
        else
        {
            _703 = _362;
            break;
        }
    } while(false);
    out_var_SV_Target = _703;
}


