// Reflections 

#version 460
 #extension GL_EXT_multiview : enable
// Vk3DVision SinglePass
///////////////////////////////////////////////////////////////////////////////

#define STEREO 1

#define VIEW gl_ViewIndex

// Stereo and Mono Texture Sampling
#if(STEREO == 1)
	#define SAMPLER_2D sampler2DArray
	#define USAMPLER_2D usampler2DArray
	#define ISAMPLER_2D isampler2DArray
	#define IMAGE_2D image2DArray
	#define UIMAGE_2D uimage2DArray
	#define UTEXTURE_2D utexture2DArray
	#define TEXTURE_2D texture2DArray
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, vec3(uv, VIEW))
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, vec3(uv, VIEW), bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, ivec3(pos, VIEW))
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, ivec3(pos, VIEW), result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, ivec3(uv, VIEW), lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, ivec3(uv, VIEW), lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, vec3(uv, VIEW), lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, vec3(uv, VIEW), lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, vec3(uv, VIEW), comp)
	#define TEXTURE_GATHER1(Sampler, uv) textureGather(Sampler, vec3(uv, VIEW))
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, vec3(uv, VIEW), offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod).xy
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, vec3(P, VIEW), dPdx, dPdy)
#else
	#define SAMPLER_2D sampler2D
	#define USAMPLER_2D usampler2D
	#define ISAMPLER_2D isampler2D
	#define IMAGE_2D image2D
	#define UIMAGE_2D uimage2D
	#define UTEXTURE_2D utexture2D
	#define TEXTURE_2D texture2D
	#define SAMPLE_TEXTURE(Sampler, uv) texture(Sampler, uv)
	#define SAMPLE_TEXTURE1(Sampler, uv, bias) texture(Sampler, uv, bias)
	#define IMAGE_LOAD(Image2D, pos) imageLoad(Image2D, pos)
	#define IMAGE_STORE(Image2D, pos, result) imageStore(Image2D, pos, result)
	#define TEXEL_FETCH(Sampler, uv, lod) texelFetch(Sampler, uv, lod)
	#define TEXEL_FETCH_OFFSET(Sampler, uv, lod, offset) texelFetchOffset(Sampler, uv, lod, offset)
	#define TEXTURE_LOD(Sampler, uv, lod) textureLod(Sampler, uv, lod)
	#define TEXTURE_LOD_OFFSET(Sampler, uv, lod, offset) textureLodOffset(Sampler, uv, lod, offset)
	#define TEXTURE_GATHER(Sampler, uv, comp) textureGather(Sampler, uv, comp)
	#define TEXTURE_GATHER1(Sampler, uv, comp) textureGather(Sampler, uv)
	#define TEXTURE_GATHER_OFFSET(Sampler, uv, offset, comp) textureGatherOffset(Sampler, uv, offset, comp)
	#define TEXTURE_GATHER_OFFSETS(Sampler, uv, offset) textureGatherOffsets(Sampler, uv, offset)
	#define TEXTURE_GATHER_OFFSETS1(Sampler, uv, offset, comp) textureGatherOffsets(Sampler, uv, offset, comp)
	#define TEXTURE_SIZE(Sampler, lod) textureSize(Sampler, lod)
	#define TEXTURE_GRAD(Sampler, P, dPdx, dPdy) textureGrad(Sampler, P, dPdx, dPdy)
#endif 
 ///////////////////////////////////////////////////////////////////////////////
#extension GL_EXT_nonuniform_qualifier : require
#extension GL_EXT_scalar_block_layout : require
#extension GL_EXT_samplerless_texture_functions : require


// Vk3DVision (http://3dsurroundgaming.com/) - Stereo Injection
struct Vk3DStereo
{
	vec4 stereo;
	vec4 custom_params;
};
layout(set = 0, binding = 15, std140) uniform Vk3DParams
{
	Vk3DStereo vk3d_params[2];
};

layout(set = 2, binding = 0, scalar) uniform type_FrameConstants
{
    layout(row_major) mat4 g_xView;
    layout(row_major) mat4 g_xProj;
    layout(row_major) mat4 g_xViewProj;
    layout(row_major) mat4 g_xInvView;
    layout(row_major) mat4 g_xInvProj;
    layout(row_major) mat4 g_xInvViewProj;
    vec3 g_xCamPos;
    float g_fScaledTime;
    vec4 g_xTwoOverScreenDimensions;
    vec2 g_xCameraTanHalfFOV;
    vec2 g_xCameraFarClipValues;
    vec2 g_xRcpScreenDims;
    vec2 g_xDepthProjViewUnpack;
    vec4 g_axViewFrustumPlanes[6];
    ivec4 g_xScreenDimensions;
    vec4 g_xTessCutoffAndFactors;
    float g_fTessMaxDist;
    float g_fTessRcpRange;
    float g_fDisplacementScale;
    float g_fBackFaceCutoff;
    float g_fFoliageFadeInvDistance;
    float g_fFoliageFadePower;
    uint g_bFoliageFadeEnabled;
    float g_fEnvironmentDecalBias;
    float g_fInstanceDecalBias;
    vec3 g_xWindDirection;
    float g_fWindStrength;
    vec3 g_xSubsurfaceScatteringWeight;
    float g_fGlobalSamplerBias;
    vec3 g_xDeformationWindowCentre;
    float g_fDeformationWindowSize;
    float g_fDeformationUVDensity;
    float g_fDeformationNormalIntensity;
    float g_fRainfallIntensity;
    float g_fRainBaseWetness;
    vec3 g_xRainDirection;
} FrameConstants;

layout(set = 1, binding = 7, std140) uniform type_ProjectedCloudParams
{
    uint g_bSunlightCloudsEnabled;
    float g_fSunlightCloudsVelocityX;
    float g_fSunlightCloudsVelocityY;
    float g_fSunlightCloudsScale;
    vec4 g_xSunlightCloudsProjectionDir;
} ProjectedCloudParams;

layout(set = 1, binding = 9, std140) uniform type_SunlightParams
{
    vec3 g_xSunLightDir;
    float g_fCSMStrength;
    vec4 g_xLightColour;
} SunlightParams;

layout(set = 0, binding = 2) uniform sampler g_xBilinearWrap;
layout(set = 1, binding = 50) uniform TEXTURE_2D g_xLighting_NormalBuffer;
layout(set = 1, binding = 51) uniform TEXTURE_2D g_xLighting_AlbedoBuffer;
layout(set = 1, binding = 52) uniform TEXTURE_2D g_xLighting_PBRPropertiesBuffer;
layout(set = 1, binding = 53) uniform TEXTURE_2D g_xLighting_DepthStencil;
layout(set = 1, binding = 100) uniform TEXTURE_2D g_xLighting_Clouds;
layout(set = 1, binding = 54) uniform TEXTURE_2D g_xShadowMask;

layout(location = 0) out vec4 out_var_SV_Target;

vec3 _121;

void main()
{
    vec2 _125 = floor(gl_FragCoord.xy);
    ivec2 _132 = ivec3(int(_125.x), int(_125.y), 0).xy;
    float _140 = FrameConstants.g_xDepthProjViewUnpack.x / (FrameConstants.g_xDepthProjViewUnpack.y - TEXEL_FETCH(g_xLighting_DepthStencil, _132, 0).x);
    vec4 _160 = TEXEL_FETCH(g_xLighting_NormalBuffer, _132, 0);
    vec4 _162 = TEXEL_FETCH(g_xLighting_AlbedoBuffer, _132, 0);
    vec4 _164 = TEXEL_FETCH(g_xLighting_PBRPropertiesBuffer, _132, 0);
    vec3 _168 = normalize((_160.xyz * 2.0) - vec3(1.0));
    vec3 _169 = _162.xyz;
    float _170 = _164.y;
    float _171 = _162.w;
    float _172 = _164.x;
    vec2 _175 = (_164.zw * 2.0) - vec2(1.0);
    vec2 _177 = _175.xy;
    float _181 = sqrt(clamp(1.0 - dot(_177, _177), 0.0, 1.0));
    vec3 _186 = vec3(_175.x, _175.y, _121.z);
    _186.z = (_160.w != 0.0) ? _181 : (-_181);
    vec4 _188 = TEXEL_FETCH(g_xShadowMask, _132, 0);
    float _189 = _188.x;
    vec4 _190 = vec4(0.0, 0.0, 0.0, 1.0);
    _190.w = _189;
    vec4 _487;
    if (_189 > 0.0)
    {
        //vec3 _196 = (vec4((FrameConstants.g_xCameraTanHalfFOV * _140) * (((_125.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _140, 1.0) * FrameConstants.g_xInvView).xyz;

        vec4 temp = vec4((FrameConstants.g_xCameraTanHalfFOV * _140) * (((_125.xy + FrameConstants.g_xTwoOverScreenDimensions.zw) * FrameConstants.g_xTwoOverScreenDimensions.xy) - vec2(1.0)), _140, 1.0);
    
		temp.x -= vk3d_params[VIEW].stereo.x * (_140 - vk3d_params[VIEW].stereo.y) * FrameConstants.g_xInvProj[0][0];	
		vec3 _196 = (temp * FrameConstants.g_xInvView).xyz;
                        
        vec3 _198 = normalize(FrameConstants.g_xCamPos - _196);
        float _205 = (_171 != 0.01960784383118152618408203125) ? _172 : 0.0;
        vec3 _207 = normalize(_198 + SunlightParams.g_xSunLightDir);
        float _209 = clamp(dot(_168, SunlightParams.g_xSunLightDir), 0.0, 1.0);
        float _211 = clamp(dot(_168, _198), 0.0, 1.0);
        float _213 = clamp(dot(_168, _207), 0.0, 1.0);
        float _215 = clamp(dot(_198, _207), 0.0, 1.0);
        bool _216 = _171 == 0.01568627543747425079345703125;
        vec3 _253;
        vec3 _254;
        vec3 _255;
        if (_216)
        {
            _253 = vec3(0.046999998390674591064453125);
            _254 = vec3(_209);
            _255 = _169;
        }
        else
        {
            vec3 _250;
            vec3 _251;
            vec3 _252;
            if (_171 == 0.007843137718737125396728515625)
            {
                _250 = vec3(0.0280000008642673492431640625);
                _251 = vec3(_209);
                _252 = _169;
            }
            else
            {
                vec3 _247;
                vec3 _248;
                vec3 _249;
                if (_171 == 0.0039215688593685626983642578125)
                {
                    _247 = vec3(0.0280000008642673492431640625);
                    _248 = vec3((clamp((clamp(dot(SunlightParams.g_xSunLightDir, -_168), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(SunlightParams.g_xSunLightDir, -_198), 0.0, 1.0), 7.0), 0.0, 1.0) * _205) + _209);
                    _249 = _169;
                }
                else
                {
                    vec3 _243 = vec3(_205);
                    _247 = mix(vec3(0.039999999105930328369140625), _169, _243);
                    _248 = vec3(_209);
                    _249 = mix(_169, vec3(0.0), _243);
                }
                _250 = _247;
                _251 = _248;
                _252 = _249;
            }
            _253 = _250;
            _254 = _251;
            _255 = _252;
        }
        vec3 _256 = _254 * _255;
        float _257 = max(_170, 0.0500000007450580596923828125);
        float _258 = _257 * _257;
        vec3 _354;
        if (_216)
        {
            float _265 = clamp(1.0 - _170, 0.0, 1.0);
            float _267 = ((_172 * 2.0) - 1.0) * 0.300000011920928955078125;
            float _273 = pow(_265, 8.0) * 500.0;
            float _276 = pow(_265, 6.0) * 2000.0;
            float _290 = dot(normalize(_186 + (_168 * ((-0.0500000007450580596923828125) + _267))), _207);
            float _310 = dot(normalize(_186 + (_168 * ((0.0500000007450580596923828125 + _267) - (_265 * 0.0500000007450580596923828125)))), _207);
            _354 = _256 + ((((vec3(((smoothstep(-1.0, 0.0, _290) * pow(sqrt(clamp(1.0 - (_290 * _290), 0.0, 1.0)), 20.0 + _273)) * ((((_172 * 0.89999997615814208984375) + 0.100000001490116119384765625) * 0.1500000059604644775390625) * (0.100000001490116119384765625 + _265))) * ((_273 + 28.0) * 0.03978873789310455322265625)) * (_169 / vec3(max(dot(_169, vec3(0.300000011920928955078125, 0.589999973773956298828125, 0.10999999940395355224609375)), 0.001000000047497451305389404296875)))) + vec3(((smoothstep(-1.0, 0.0, _310) * pow(sqrt(clamp(1.0 - (_310 * _310), 0.0, 1.0)), 50.0 + _276)) * ((((_172 * 0.5) + 0.5) * 0.014999999664723873138427734375) * _265)) * ((_276 + 58.0) * 0.03978873789310455322265625))) * _209) * 1.0);
        }
        else
        {
            float _334 = 2.0 - _258;
            float _341 = _258 * _258;
            float _345 = (((_213 * _341) - _213) * _213) + 1.0;
            _354 = _256 + (((((vec3(1.0) - _253) * exp2((((-5.554729938507080078125) * _215) - 6.9831600189208984375) * _215)) + _253) * (((1.0 / (((_211 * _334) + _258) * ((_209 * _334) + _258))) * (_341 * (1.0 / (_345 * _345)))) * _209)) * 1.0);
        }
        vec3 _403;
        if (_171 == 0.01960784383118152618408203125)
        {
            bool _358 = _172 > 0.4980392158031463623046875;
            vec3 _376 = sqrt(abs(_255));
            float _390 = 0.5 / _258;
            _403 = mix(_354, (_256 + (((((vec3(1.0) - _376) * exp2((((-5.554729938507080078125) * _215) - 6.9831600189208984375) * _215)) + _376) * ((1.0 / ((4.0 * ((_209 + _211) - (_209 * _211))) + 9.9999996826552253889678874634872e-21)) * ((1.0 + _390) * pow(1.0 - (_213 * _213), _390)))) * _209)) + (_255 * (clamp((clamp(dot(SunlightParams.g_xSunLightDir, -_168), 0.0, 1.0) * 0.4000000059604644775390625) + pow(clamp(dot(SunlightParams.g_xSunLightDir, -_198), 0.0, 1.0), 7.0), 0.0, 1.0) * (_358 ? 0.5 : 0.0))), vec3((_358 ? (_172 - 0.501960813999176025390625) : _172) * 2.007874011993408203125));
        }
        else
        {
            _403 = _354;
        }
        vec3 _405 = min(_403 * SunlightParams.g_xLightColour.xyz, vec3(1000.0));
        float _457;
        if (ProjectedCloudParams.g_bSunlightCloudsEnabled != 0u)
        {
            float _456;
            if (abs(ProjectedCloudParams.g_xSunlightCloudsProjectionDir.y) > 9.9999999747524270787835121154785e-07)
            {
                float _421 = (vec3(0.0, -10000.0, 0.0) - _196).y / ProjectedCloudParams.g_xSunlightCloudsProjectionDir.y;
                float _455;
                if (_421 > 0.0)
                {
                    vec3 _427 = _196 + (ProjectedCloudParams.g_xSunlightCloudsProjectionDir.xyz * _421);
                    vec2 _436 = _427.xz;
                    _436.x = _427.x + (FrameConstants.g_fScaledTime * ProjectedCloudParams.g_fSunlightCloudsVelocityX);
                    vec2 _442 = _436;
                    _442.y = _427.z + (FrameConstants.g_fScaledTime * ProjectedCloudParams.g_fSunlightCloudsVelocityY);
                    vec2 _446 = _442 / vec2(ProjectedCloudParams.g_fSunlightCloudsScale);
                    vec2 _449 = _446;
                    _449.x = _446.x * (-1.0);
                    _455 = TEXTURE_LOD(SAMPLER_2D(g_xLighting_Clouds, g_xBilinearWrap), _449, 0.0).x;
                }
                else
                {
                    _455 = 1.0;
                }
                _456 = _455;
            }
            else
            {
                _456 = 1.0;
            }
            _457 = _456;
        }
        else
        {
            _457 = 1.0;
        }
        vec4 _459 = vec4(_405.x, _405.y, _405.z, _190.w);
        _459.w = _189 * _457;
        bool _461 = _171 == 0.007843137718737125396728515625;
        vec3 _472;
        if (_461)
        {
            _472 = _459.xyz + (((_169 * vec3(0.0194447897374629974365234375, 1.6805228369776159524917602539063e-05, 3.1907067266079280898338765837252e-11)) * clamp(0.300000011920928955078125 + dot(SunlightParams.g_xSunLightDir, -_168), 0.0, 1.0)) * SunlightParams.g_xLightColour.xyz);
        }
        else
        {
            _472 = _459.xyz;
        }
        vec3 _485;
        if (_461 && (_140 < 3.0))
        {
            _485 = _472.xyz * mix(FrameConstants.g_xSubsurfaceScatteringWeight, vec3(1.0), vec3(max(_140 - 2.0, 0.0)));
        }
        else
        {
            _485 = _472.xyz;
        }
        _487 = vec4(_485.x, _485.y, _485.z, _459.w);
    }
    else
    {
        _487 = _190;
    }
    out_var_SV_Target = _487;
}



